@@ -19,6 +19,10 @@
 #ifndef _SW_ASYNC_H_
 #define _SW_ASYNC_H_
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #ifndef O_DIRECT
 #define O_DIRECT         040000
 #endif
@@ -92,4 +96,8 @@ int swAioGcc_init(int max_aio_events);
 int swAioLinux_init(int max_aio_events);
 #endif
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* _SW_ASYNC_H_ */
