@@ -68,6 +68,7 @@ enum swErrorCode
      */
     SW_ERROR_HTTP2_STREAM_ID_TOO_BIG = 3001,
     SW_ERROR_HTTP2_STREAM_NO_HEADER,
+    SW_ERROR_HTTP2_STREAM_NOT_FOUND,
 
     /**
      * AIO
@@ -127,6 +128,8 @@ enum swErrorCode
     SW_ERROR_CO_STD_THREAD_LINK_ERROR,
     SW_ERROR_CO_DISABLED_MULTI_THREAD,
 
+    SW_ERROR_CO_OUT_OF_COROUTINE,
+
 };
 
 void swoole_throw_error(enum swErrorCode code);
