@@ -71,10 +71,10 @@ typedef struct
 
 enum swWebsocketCode
 {
-    WEBSOCKET_OPCODE_CONTINUATION_FRAME = 0x0,
-    WEBSOCKET_OPCODE_TEXT_FRAME = 0x1,
-    WEBSOCKET_OPCODE_BINARY_FRAME = 0x2,
-    WEBSOCKET_OPCODE_CONNECTION_CLOSE = 0x8,
+    WEBSOCKET_OPCODE_CONTINUATION = 0x0,
+    WEBSOCKET_OPCODE_TEXT = 0x1,
+    WEBSOCKET_OPCODE_BINARY = 0x2,
+    WEBSOCKET_OPCODE_CLOSE = 0x8,
     WEBSOCKET_OPCODE_PING = 0x9,
     WEBSOCKET_OPCODE_PONG = 0xa,
 
