@@ -29,6 +29,22 @@ static sw_inline zend_bool Z_BVAL_P(zval *v)
 }
 
 //----------------------------------Array API------------------------------------
+
+static sw_inline int add_assoc_ulong_safe(zval *arg, const char *key, zend_ulong value)
+{
+    if (likely(value <= ZEND_LONG_MAX))
+    {
+        return add_assoc_long(arg, key, value);
+    }
+    else
+    {
+        char buf[MAX_LENGTH_OF_LONG + 1];
+        memset((char *) buf, 0, MAX_LENGTH_OF_LONG + 1);
+        sprintf((char *) buf, ZEND_ULONG_FMT, value);
+        return add_assoc_string(arg, key, buf);
+    }
+}
+
 #define sw_add_assoc_stringl(__arg, __key, __str, __length, __duplicate)   add_assoc_stringl_ex(__arg, __key, strlen(__key), __str, __length)
 static sw_inline int sw_add_assoc_stringl_ex(zval *arg, const char *key, size_t key_len, char *str, size_t length, int __duplicate)
 {
@@ -89,7 +105,7 @@ static sw_inline char* sw_php_url_encode(char *value, size_t value_len, int* ext
 static sw_inline int sw_call_user_function_ex(HashTable *function_table, zval** object_pp, zval *function_name, zval **retval_ptr_ptr, uint32_t param_count, zval ***params, int no_separation, HashTable* ymbol_table)
 {
     zval real_params[SW_PHP_MAX_PARAMS_NUM];
-    int i = 0;
+    uint32_t i = 0;
     for (; i < param_count; i++)
     {
         real_params[i] = **params[i];
@@ -104,7 +120,7 @@ static sw_inline int sw_call_user_function_ex(HashTable *function_table, zval**
 static sw_inline int sw_call_user_function_fast(zval *function_name, zend_fcall_info_cache *fci_cache, zval **retval_ptr_ptr, uint32_t param_count, zval ***params)
 {
     zval real_params[SW_PHP_MAX_PARAMS_NUM];
-    int i = 0;
+    uint32_t i = 0;
     for (; i < param_count; i++)
     {
         real_params[i] = **params[i];
@@ -115,7 +131,7 @@ static sw_inline int sw_call_user_function_fast(zval *function_name, zend_fcall_
 
     zend_fcall_info fci;
     fci.size = sizeof(fci);
-#if PHP_MINOR_VERSION == 0
+#if PHP_MAJOR_VERSION == 7 && PHP_MINOR_VERSION == 0
     fci.function_table = EG(function_table);
     fci.symbol_table = NULL;
 #endif
@@ -168,7 +184,6 @@ static sw_inline int sw_call_user_function_fast(zval *function_name, zend_fcall_
     if (ZVAL_IS_NULL(&__retval)) *(retval) = NULL;\
     else *(retval) = &__retval;
 
-#define SWOOLE_GET_TSRMLS
 #define SW_ZVAL_STRINGL(z, s, l, dup)         ZVAL_STRINGL(z, s, l)
 #define SW_ZVAL_STRING(z,s,dup)               ZVAL_STRING(z,s)
 #define sw_smart_str                          smart_string
@@ -291,7 +306,7 @@ static inline int sw_zend_register_class_alias(const char *name, zend_class_entr
 
     zend_string *_interned_name = zend_new_interned_string(_name);
 
-#if PHP_MINOR_VERSION > 2
+#if PHP_VERSION_ID >= 70300
     return zend_register_class_alias_ex(_interned_name->val, _interned_name->len, ce, 1);
 #else
     return zend_register_class_alias_ex(_interned_name->val, _interned_name->len, ce);
