@@ -17,6 +17,11 @@
 #ifndef PHP_SWOOLE_H
 #define PHP_SWOOLE_H
 
+// C++ build format macros must defined earlier
+#ifdef __cplusplus
+#define __STDC_FORMAT_MACROS
+#endif
+
 #include "php.h"
 #include "php_ini.h"
 #include "php_globals.h"
@@ -28,29 +33,41 @@
 #include "zend_interfaces.h"
 #include "zend_exceptions.h"
 #include "zend_variables.h"
-#include <ext/date/php_date.h>
-#include <ext/standard/url.h>
-#include <ext/standard/info.h>
-#include <ext/standard/php_array.h>
-#include <ext/standard/basic_functions.h>
-#include <ext/standard/php_http.h>
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+// zend iterator interface
+#if PHP_VERSION_ID < 70200
+#ifdef HAVE_PCRE
+#include "ext/spl/spl_iterators.h"
+#define zend_ce_countable spl_ce_Countable
+#define SW_HAVE_COUNTABLE 1
+#endif
+#else
+#define SW_HAVE_COUNTABLE 1
+#endif
+
 #ifdef SW_STATIC_COMPILATION
 #include "php_config.h"
 #endif
 
 #include "swoole.h"
-#include "Server.h"
-#include "Client.h"
+#include "server.h"
+#include "client.h"
 #include "async.h"
 
 BEGIN_EXTERN_C()
+#include <ext/date/php_date.h>
+#include <ext/standard/url.h>
+#include <ext/standard/info.h>
+#include <ext/standard/php_array.h>
+#include <ext/standard/php_var.h>
+#include <ext/standard/basic_functions.h>
+#include <ext/standard/php_http.h>
 
-#define PHP_SWOOLE_VERSION  "4.0.4"
+#define PHP_SWOOLE_VERSION  "4.1.2"
 #define PHP_SWOOLE_CHECK_CALLBACK
 #define PHP_SWOOLE_ENABLE_FASTCALL
 #define PHP_SWOOLE_CLIENT_USE_POLL
@@ -61,6 +78,7 @@ BEGIN_EXTERN_C()
 
 #define SW_HOST_SIZE  128
 
+extern PHPAPI int php_array_merge(HashTable *dest, HashTable *src);
 typedef struct
 {
     uint16_t port;
@@ -98,6 +116,11 @@ extern __thread swoole_object_array swoole_objects;
 extern swoole_object_array swoole_objects;
 #endif
 
+// Solaris doesn't have PTRACE_ATTACH
+#if defined(HAVE_PTRACE) && defined(__sun)
+#undef HAVE_PTRACE
+#endif
+
 //#define SW_USE_PHP        1
 #define SW_CHECK_RETURN(s)         if(s<0){RETURN_FALSE;}else{RETURN_TRUE;}return
 #define SW_LOCK_CHECK_RETURN(s)    if(s==0){RETURN_TRUE;}else{\
@@ -109,14 +132,6 @@ extern swoole_object_array swoole_objects;
 #define swoole_php_sys_error(level, fmt_str, ...)  if (SWOOLE_G(display_errors)) php_error_docref(NULL TSRMLS_CC, level, fmt_str" Error: %s[%d].", ##__VA_ARGS__, strerror(errno), errno)
 #define swoole_efree(p)  if (p) efree(p)
 
-#if defined(SW_ASYNC_MYSQL)
-#if defined(SW_HAVE_MYSQLI) && defined(SW_HAVE_MYSQLND)
-#else
-#error "Enable async_mysql support, require mysqli and mysqlnd."
-#undef SW_ASYNC_MYSQL
-#endif
-#endif
-
 #ifdef SW_USE_OPENSSL
 #ifndef HAVE_OPENSSL
 #error "Enable openssl support, require openssl library."
@@ -194,9 +209,7 @@ typedef struct
 {
     zval *callbacks[PHP_SERVER_CALLBACK_NUM];
     zend_fcall_info_cache *caches[PHP_SERVER_CALLBACK_NUM];
-#if PHP_MAJOR_VERSION >= 7
     zval _callbacks[PHP_SERVER_CALLBACK_NUM];
-#endif
     zval *setting;
     swServer *serv;
 } swoole_server_port_property;
@@ -218,7 +231,6 @@ enum php_swoole_fd_type
     PHP_SWOOLE_FD_REDIS_CORO,
     PHP_SWOOLE_FD_POSTGRESQL,
     PHP_SWOOLE_FD_SOCKET,
-    PHP_SWOOLE_FD_CHAN_PIPE,
 #endif
     /**
      * for Co::fread/Co::fwrite
@@ -251,9 +263,7 @@ extern zend_class_entry *swoole_exception_class_entry_ptr;
 
 extern zval *php_sw_server_callbacks[PHP_SERVER_CALLBACK_NUM];
 extern zend_fcall_info_cache *php_sw_server_caches[PHP_SERVER_CALLBACK_NUM];
-#if PHP_MAJOR_VERSION >= 7
 extern zval _php_sw_server_callbacks[PHP_SERVER_CALLBACK_NUM];
-#endif
 
 PHP_MINIT_FUNCTION(swoole);
 PHP_MSHUTDOWN_FUNCTION(swoole);
@@ -309,7 +319,6 @@ PHP_METHOD(swoole_server, connection_info);
 PHP_METHOD(swoole_server, getReceivedTime);
 #endif
 
-#ifdef HAVE_PCRE
 PHP_METHOD(swoole_connection_iterator, count);
 PHP_METHOD(swoole_connection_iterator, rewind);
 PHP_METHOD(swoole_connection_iterator, next);
@@ -321,7 +330,6 @@ PHP_METHOD(swoole_connection_iterator, offsetGet);
 PHP_METHOD(swoole_connection_iterator, offsetSet);
 PHP_METHOD(swoole_connection_iterator, offsetUnset);
 PHP_METHOD(swoole_connection_iterator, __destruct);
-#endif
 
 #ifdef SWOOLE_SOCKETS_SUPPORT
 PHP_METHOD(swoole_server, getSocket);
@@ -376,9 +384,7 @@ void swoole_destory_table(zend_resource *rsrc TSRMLS_DC);
 void swoole_server_port_init(int module_number TSRMLS_DC);
 void swoole_async_init(int module_number TSRMLS_DC);
 void swoole_table_init(int module_number TSRMLS_DC);
-#ifdef SW_USE_PHPX
 void swoole_runtime_init(int module_number TSRMLS_DC);
-#endif
 void swoole_lock_init(int module_number TSRMLS_DC);
 void swoole_atomic_init(int module_number TSRMLS_DC);
 void swoole_client_init(int module_number TSRMLS_DC);
@@ -449,14 +455,14 @@ zval* php_swoole_task_unpack(swEventData *task_result TSRMLS_DC);
 
 static sw_inline void* swoole_get_object(zval *object)
 {
-    int handle = sw_get_object_handle(object);
+    uint32_t handle = sw_get_object_handle(object);
     assert(handle < swoole_objects.size);
     return swoole_objects.array[handle];
 }
 
 static sw_inline void* swoole_get_property(zval *object, int property_id)
 {
-    int handle = sw_get_object_handle(object);
+    uint32_t handle = sw_get_object_handle(object);
     if (handle >= swoole_objects.property_size[property_id])
     {
         return NULL;
@@ -491,10 +497,8 @@ int php_swoole_client_onPackage(swConnection *conn, char *data, uint32_t length)
 void php_swoole_onTimeout(swTimer *timer, swTimer_node *tnode);
 void php_swoole_onInterval(swTimer *timer, swTimer_node *tnode);
 
-#if PHP_MAJOR_VERSION >= 7
 PHPAPI zend_string* php_swoole_serialize(zval *zvalue);
 PHPAPI int php_swoole_unserialize(void *buffer, size_t len, zval *return_value, zval *object_args, long flag);
-#endif
 
 #ifdef SW_COROUTINE
 int php_coroutine_reactor_can_exit(swReactor *reactor);
@@ -503,7 +507,7 @@ int php_coroutine_reactor_can_exit(swReactor *reactor);
 static sw_inline zval* php_swoole_server_get_callback(swServer *serv, int server_fd, int event_type)
 {
     swListenPort *port = (swListenPort *) serv->connection_list[server_fd].object;
-    if (port == NULL)
+    if (!port)
     {
         swWarn("invalid server_fd[%d].", server_fd);
         return NULL;
@@ -528,6 +532,11 @@ static sw_inline zval* php_swoole_server_get_callback(swServer *serv, int server
 static sw_inline zend_fcall_info_cache* php_swoole_server_get_cache(swServer *serv, int server_fd, int event_type)
 {
     swListenPort *port = (swListenPort *) serv->connection_list[server_fd].object;
+    if (!port)
+    {
+        swWarn("invalid server_fd[%d].", server_fd);
+        return NULL;
+    }
     swoole_server_port_property *property = (swoole_server_port_property *) port->ptr;
     if (!property)
     {
@@ -600,6 +609,7 @@ ZEND_BEGIN_MODULE_GLOBALS(swoole)
     zend_bool use_namespace;
     zend_bool use_shortname;
     zend_bool fast_serialize;
+    zend_bool enable_coroutine;
     long socket_buffer_size;
     php_swoole_req_status req_status;
     swLinkedList *rshutdown_functions;
@@ -613,6 +623,7 @@ extern ZEND_DECLARE_MODULE_GLOBALS(swoole);
 #define SWOOLE_G(v) (swoole_globals.v)
 #endif
 
+#define SWOOLE_RAW_DEFINE(constant)    REGISTER_LONG_CONSTANT(#constant, constant, CONST_CS | CONST_PERSISTENT)
 #define SWOOLE_DEFINE(constant)    REGISTER_LONG_CONSTANT("SWOOLE_"#constant, SW_##constant, CONST_CS | CONST_PERSISTENT)
 
 #define SWOOLE_INIT_CLASS_ENTRY(ce, name, name_ns, methods) \
