@@ -21,6 +21,16 @@
 #include <sys/file.h>
 #include <sys/stat.h>
 
+#ifdef __sun
+#include <fcntl.h>
+#define LOCK_SH             F_RDLCK
+#define LOCK_EX             F_WRLCK
+#define LOCK_UN             F_UNLCK
+struct flock lock;
+#define flock(fildes, cmd) \
+    lock.l_type = (cmd), fcntl(fildes, F_SETLK, &lock)
+#endif
+
 swAsyncIO SwooleAIO;
 swPipe swoole_aio_pipe;
 
