@@ -27,10 +27,8 @@
 
 typedef struct
 {
-#if PHP_MAJOR_VERSION >= 7
     zval _callback;
     zval _filename;
-#endif
     zval *callback;
     zval *filename;
     int fd;
@@ -49,10 +47,8 @@ typedef struct
 
 typedef struct
 {
-#if PHP_MAJOR_VERSION >= 7
     zval _callback;
     zval _domain;
-#endif
     zval *callback;
     zval *domain;
 #ifdef SW_COROUTINE
@@ -156,7 +152,6 @@ void php_swoole_check_aio()
 
 static void php_swoole_dns_callback(char *domain, swDNSResolver_result *result, void *data)
 {
-    SWOOLE_GET_TSRMLS;
     dns_request *req = data;
     zval *retval = NULL;
     zval *zaddress;
@@ -209,7 +204,6 @@ static void php_swoole_dns_callback(char *domain, swDNSResolver_result *result,
 #ifdef SW_COROUTINE
 static void php_swoole_dns_callback_coro(char *domain, swDNSResolver_result *result, void *data)
 {
-    SWOOLE_GET_TSRMLS;
     dns_request *req = data;
     zval *retval = NULL;
 
@@ -544,7 +538,7 @@ PHP_FUNCTION(swoole_async_read)
     }
     if (offset >= file_stat.st_size)
     {
-        swoole_php_fatal_error(E_WARNING, "offset must be less than file_size[=%ld].", file_stat.st_size);
+        swoole_php_fatal_error(E_WARNING, "offset must be less than file_size[=%jd].", (intmax_t) file_stat.st_size);
         close(fd);
         RETURN_FALSE;
     }
@@ -1006,7 +1000,6 @@ PHP_FUNCTION(swoole_async_dns_lookup)
 
 static int process_stream_onRead(swReactor *reactor, swEvent *event)
 {
-    SWOOLE_GET_TSRMLS;
 
     process_stream *ps = event->socket->object;
     char *buf = ps->buffer->str + ps->buffer->length;
