@@ -32,19 +32,15 @@
 # define GC_DELREF(ref) --GC_REFCOUNT(ref)
 #endif/*}}}*/
 
-/* output globals */
-#define SWOG ((zend_output_globals *) &OG(handlers))
-
 #define TASK_SLOT \
     ((int)((ZEND_MM_ALIGNED_SIZE(sizeof(coro_task)) + ZEND_MM_ALIGNED_SIZE(sizeof(zval)) - 1) / ZEND_MM_ALIGNED_SIZE(sizeof(zval))))
 #define SWCC(x) sw_current_context->x
 
 coro_global COROG;
-static coro_task* sw_get_current_task();
 static void sw_coro_func(void *);
 static zend_bool is_xdebug_started = 0;
 
-#if PHP_MAJOR_VERSION >= 7 && PHP_MINOR_VERSION >= 2
+#if PHP_VERSION_ID >= 70200
 static inline void sw_vm_stack_init(void)
 {
     uint32_t size = COROG.stack_size;
@@ -69,11 +65,13 @@ int coro_init(TSRMLS_D)
     {
         is_xdebug_started = 1;
     }
+    //save init vm
     COROG.origin_vm_stack = EG(vm_stack);
     COROG.origin_vm_stack_top = EG(vm_stack_top);
     COROG.origin_vm_stack_end = EG(vm_stack_end);
 
     COROG.coro_num = 0;
+    COROG.peak_coro_num = 0;
     if (COROG.max_coro_num <= 0)
     {
         COROG.max_coro_num = DEFAULT_MAX_CORO_NUM;
@@ -84,10 +82,81 @@ int coro_init(TSRMLS_D)
     }
 
     COROG.active = 1;
-    coroutine_set_close(sw_coro_close);
+    //set functions
+    coroutine_set_onYield(internal_coro_yield);
+    coroutine_set_onResume(internal_coro_resume);
+    coroutine_set_onClose(sw_coro_close);
     return 0;
 }
 
+static void resume_php_stack(coro_task *task)
+{
+    COROG.call_stack[COROG.call_stack_size++] = task;
+    COROG.current_coro = task;
+    swTraceLog(SW_TRACE_COROUTINE,"sw_coro_resume coro id %d", COROG.current_coro->cid);
+    task->state = SW_CORO_RUNNING;
+    EG(current_execute_data) = task->yield_execute_data;
+    EG(vm_stack) = task->yield_stack;
+    EG(vm_stack_top) = task->yield_vm_stack_top;
+    EG(vm_stack_end) = task->yield_vm_stack_end;
+}
+
+static void save_php_stack(coro_task *task)
+{
+    COROG.call_stack_size--;
+    swTraceLog(SW_TRACE_COROUTINE,"coro_yield coro id %d", task->cid);
+    task->state = SW_CORO_YIELD;
+    task->is_yield = 1;
+    //save vm stack
+    task->yield_execute_data = EG(current_execute_data);
+    task->yield_stack = EG(vm_stack);
+    task->yield_vm_stack_top = EG(vm_stack_top);
+    task->yield_vm_stack_end = EG(vm_stack_end);
+    //restore vm stack
+    EG(vm_stack) = task->origin_stack;
+    EG(vm_stack_top) = task->origin_vm_stack_top;
+    EG(vm_stack_end) = task->origin_vm_stack_end;
+}
+void internal_coro_resume(void *arg)
+{
+    coro_task *task = (coro_task *)arg;
+    resume_php_stack(task);
+    // main OG
+    if (OG(handlers).elements)
+    {
+        php_output_deactivate(); // free main
+        if (!task->current_coro_output_ptr)
+        {
+            php_output_activate(); // reset output
+        }
+    }
+    // resume output control global
+    if (task->current_coro_output_ptr)
+    {
+        memcpy(SWOG, task->current_coro_output_ptr, sizeof(zend_output_globals));
+        efree(task->current_coro_output_ptr);
+        task->current_coro_output_ptr = NULL;
+    }
+    swTraceLog(SW_TRACE_COROUTINE, "cid=%d", task->cid);
+}
+
+void internal_coro_yield(void *arg)
+{
+    coro_task *task = (coro_task *)arg;
+    save_php_stack(task);
+    // save output control global
+    if (OG(active))
+    {
+        zend_output_globals *coro_output_globals_ptr = (zend_output_globals *) emalloc(sizeof(zend_output_globals));
+        memcpy(coro_output_globals_ptr, SWOG, sizeof(zend_output_globals));
+        task->current_coro_output_ptr = coro_output_globals_ptr;
+        php_output_activate(); // reset output
+    }
+    else
+    {
+        task->current_coro_output_ptr = NULL;
+    }
+}
 
 void coro_check(TSRMLS_D)
 {
@@ -99,12 +168,7 @@ void coro_check(TSRMLS_D)
 
 void coro_destroy(TSRMLS_D)
 {
-    if (COROG.chan_pipe)
-    {
-        COROG.chan_pipe->close(COROG.chan_pipe);
-        efree(COROG.chan_pipe);
-        COROG.chan_pipe = NULL;
-    }
+
 }
 
 static void sw_coro_func(void *arg)
@@ -114,12 +178,10 @@ static void sw_coro_func(void *arg)
     zval **argv = php_arg->argv;
     int argc = php_arg->argc;
     zval *retval = php_arg->retval;
-    void *post_callback = php_arg->post_callback;
-    void* params = php_arg->params;
-    int cid = coroutine_get_cid();
+    int cid = coroutine_get_current_cid();
 
+    int i;
     zend_function *func;
-    uint32_t i;
     coro_task *task;
 
     zend_vm_stack origin_vm_stack = EG(vm_stack);
@@ -136,7 +198,7 @@ static void sw_coro_func(void *arg)
     call = zend_vm_stack_push_call_frame(ZEND_CALL_TOP_FUNCTION | ZEND_CALL_ALLOCATED, func, argc,
             fci_cache->called_scope, fci_cache->object);
 
-#if PHP_MINOR_VERSION < 1
+#if PHP_VERSION_ID < 70100
     EG(scope) = func->common.scope;
 #endif
 
@@ -171,6 +233,7 @@ static void sw_coro_func(void *arg)
     task->is_yield = 0;
     task->state = SW_CORO_RUNNING;
     task->co = coroutine_get_by_id(cid);
+    coroutine_set_ptr(task->co, (void *)task);
 
     if (SwooleG.hooks[SW_GLOBAL_HOOK_ON_CORO_START])
     {
@@ -214,6 +277,11 @@ int sw_coro_create(zend_fcall_info_cache *fci_cache, zval **argv, int argc, zval
     COROG.error = 0;
     COROG.coro_num++;
 
+    if (COROG.coro_num > COROG.peak_coro_num)
+    {
+        COROG.peak_coro_num = COROG.coro_num;
+    }
+
     /**===================Before Coroutine======================**/
     zend_output_globals *coro_output_globals_ptr = NULL;
     if (OG(active)) // save the current OG
@@ -263,15 +331,7 @@ void sw_coro_save(zval *return_value, php_context *sw_current_context)
 int sw_coro_resume(php_context *sw_current_context, zval *retval, zval *coro_retval)
 {
     coro_task *task = SWCC(current_task);
-    COROG.call_stack[COROG.call_stack_size++] = task;
-    COROG.current_coro = task;
-    swTraceLog(SW_TRACE_COROUTINE,"sw_coro_resume coro id %d", COROG.current_coro->cid);
-    task->state = SW_CORO_RUNNING;
-    EG(current_execute_data) = SWCC(current_execute_data);
-    EG(vm_stack) = SWCC(current_vm_stack);
-    EG(vm_stack_top) = SWCC(current_vm_stack_top);
-    EG(vm_stack_end) = SWCC(current_vm_stack_end);
-
+    resume_php_stack(task);
     if (EG(current_execute_data)->prev_execute_data->opline->result_type != IS_UNUSED && retval)
     {
         ZVAL_COPY(SWCC(current_coro_return_value_ptr), retval);
@@ -294,8 +354,8 @@ int sw_coro_resume(php_context *sw_current_context, zval *retval, zval *coro_ret
         SWCC(current_coro_output_ptr) = NULL;
     }
 
-    swDebug("cid=%d", task->cid);
-    coroutine_resume(task->co);
+    swTraceLog(SW_TRACE_COROUTINE, "cid=%d", task->cid);
+    coroutine_resume_naked(task->co);
 
     if (unlikely(EG(exception)))
     {
@@ -310,15 +370,13 @@ int sw_coro_resume(php_context *sw_current_context, zval *retval, zval *coro_ret
 
 void sw_coro_yield()
 {
+    if (sw_get_current_cid() == -1)
+    {
+        swoole_php_fatal_error(E_ERROR, "must be called in the coroutine.");
+    }
     coro_task *task = (coro_task *) sw_get_current_task();
-    COROG.call_stack_size--;
-    swTraceLog(SW_TRACE_COROUTINE,"coro_yield coro id %d", task->cid);
-    task->state = SW_CORO_YIELD;
-    task->is_yield = 1;
-    EG(vm_stack) = task->origin_stack;
-    EG(vm_stack_top) = task->origin_vm_stack_top;
-    EG(vm_stack_end) = task->origin_vm_stack_end;
-    coroutine_yield(task->co);
+    save_php_stack(task);
+    coroutine_yield_naked(task->co);
 }
 
 void sw_coro_close()
@@ -379,7 +437,7 @@ int sw_get_current_cid()
     }
 }
 
-static coro_task* sw_get_current_task()
+coro_task* sw_get_current_task()
 {
     return (COROG.call_stack_size > 0) ? COROG.call_stack[COROG.call_stack_size - 1] : NULL;
 }
