@@ -42,6 +42,8 @@ typedef struct _coro_task
     zval *yield_vm_stack_top;
     zval *yield_vm_stack_end;
     zend_bool is_yield;
+
+    zend_output_globals *current_coro_output_ptr;
     /**
      * user coroutine
      */
@@ -75,6 +77,7 @@ typedef struct _coro_global
 {
     uint32_t coro_num;
     uint32_t max_coro_num;
+    uint32_t peak_coro_num;
     uint32_t stack_size;
     zend_vm_stack origin_vm_stack;
     zval *origin_vm_stack_top;
@@ -83,9 +86,8 @@ typedef struct _coro_global
     zend_execute_data *origin_ex;
     coro_task *current_coro;
     zend_bool active;
-    coro_task *call_stack[128];
+    coro_task *call_stack[SW_MAX_CORO_NESTING_LEVEL];
     int call_stack_size;
-    swPipe *chan_pipe;
     int error;
 } coro_global;
 
@@ -112,11 +114,17 @@ void coro_check(TSRMLS_D);
         sw_coro_resume(sw_current_context, retval, *coro_retval)
 #define coro_yield() sw_coro_yield()
 
+#define coro_use_return_value(); *(zend_uchar *) &execute_data->prev_execute_data->opline->result_type = IS_VAR;
+
+/* output globals */
+#define SWOG ((zend_output_globals *) &OG(handlers))
+
 int sw_coro_create(zend_fcall_info_cache *op_array, zval **argv, int argc, zval *retval, void *post_callback, void *param);
 void sw_coro_yield();
 void sw_coro_close();
 int sw_coro_resume(php_context *sw_current_context, zval *retval, zval *coro_retval);
 void sw_coro_save(zval *return_value, php_context *sw_php_context);
+coro_task* sw_get_current_task();
 
 int php_swoole_add_timer_coro(int ms, int cli_fd, long *timeout_id, void* param, swLinkedList_node **node TSRMLS_DC);
 int php_swoole_clear_timer_coro(long id TSRMLS_DC);
