@@ -17,9 +17,9 @@
  */
 #include "php_swoole.h"
 
-#ifdef SW_COROUTINE
 #include "swoole_coroutine.h"
 #include "async.h"
+#include "zend_builtin_functions.h"
 #include "ext/standard/file.h"
 
 typedef struct
@@ -31,6 +31,13 @@ typedef struct
     swTimer_node *timer;
 } util_socket;
 
+typedef struct
+{
+    int current_cid;
+    int index;
+    int count;
+} coroutine_iterator;
+
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
@@ -92,11 +99,18 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_writeFile, 0, 0, 2)
     ZEND_ARG_INFO(0, flags)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_getBackTrace, 0, 0, 1)
+    ZEND_ARG_INFO(0, cid)
+    ZEND_ARG_INFO(0, options)
+    ZEND_ARG_INFO(0, limit)
+ZEND_END_ARG_INFO()
+
 static PHP_METHOD(swoole_coroutine_util, set);
 static PHP_METHOD(swoole_coroutine_util, yield);
 static PHP_METHOD(swoole_coroutine_util, resume);
 static PHP_METHOD(swoole_coroutine_util, stats);
 static PHP_METHOD(swoole_coroutine_util, getuid);
+static PHP_METHOD(swoole_coroutine_util, listCoroutines);
 static PHP_METHOD(swoole_coroutine_util, sleep);
 static PHP_METHOD(swoole_coroutine_util, fread);
 static PHP_METHOD(swoole_coroutine_util, fgets);
@@ -105,12 +119,30 @@ static PHP_METHOD(swoole_coroutine_util, gethostbyname);
 static PHP_METHOD(swoole_coroutine_util, getaddrinfo);
 static PHP_METHOD(swoole_coroutine_util, readFile);
 static PHP_METHOD(swoole_coroutine_util, writeFile);
+static PHP_METHOD(swoole_coroutine_util, getBackTrace);
+
+static PHP_METHOD(swoole_coroutine_iterator, count);
+static PHP_METHOD(swoole_coroutine_iterator, rewind);
+static PHP_METHOD(swoole_coroutine_iterator, next);
+static PHP_METHOD(swoole_coroutine_iterator, current);
+static PHP_METHOD(swoole_coroutine_iterator, key);
+static PHP_METHOD(swoole_coroutine_iterator, valid);
+static PHP_METHOD(swoole_coroutine_iterator, __destruct);
+
+static PHP_METHOD(swoole_exit_exception, getFlags);
+static PHP_METHOD(swoole_exit_exception, getStatus);
 
 static swHashMap *defer_coros;
 
 static zend_class_entry swoole_coroutine_util_ce;
 static zend_class_entry *swoole_coroutine_util_class_entry_ptr;
 
+static zend_class_entry swoole_coroutine_iterator_ce;
+static zend_class_entry *swoole_coroutine_iterator_class_entry_ptr;
+
+static zend_class_entry swoole_exit_exception_ce;
+static zend_class_entry *swoole_exit_exception_class_entry_ptr;
+
 static const zend_function_entry swoole_coroutine_util_methods[] =
 {
     ZEND_FENTRY(create, ZEND_FN(swoole_coroutine_create), arginfo_swoole_coroutine_create, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
@@ -129,14 +161,104 @@ static const zend_function_entry swoole_coroutine_util_methods[] =
     PHP_ME(swoole_coroutine_util, writeFile, arginfo_swoole_coroutine_writeFile, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, gethostbyname, arginfo_swoole_coroutine_gethostbyname, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, getaddrinfo, arginfo_swoole_coroutine_getaddrinfo, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    PHP_ME(swoole_coroutine_util, getBackTrace, arginfo_swoole_coroutine_getBackTrace, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    PHP_ME(swoole_coroutine_util, listCoroutines, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_FE_END
 };
 
+static const zend_function_entry iterator_methods[] =
+{
+    PHP_ME(swoole_coroutine_iterator, rewind,      arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_coroutine_iterator, next,        arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_coroutine_iterator, current,     arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_coroutine_iterator, key,         arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_coroutine_iterator, valid,       arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_coroutine_iterator, count,       arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_coroutine_iterator, __destruct,  arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_DTOR)
+    PHP_FE_END
+};
+
+static const zend_function_entry swoole_exit_exception_methods[] =
+{
+    PHP_ME(swoole_exit_exception, getFlags, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_exit_exception, getStatus, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC)
+    PHP_FE_END
+};
+
+static user_opcode_handler_t ori_exit_handler = NULL;
+
+enum sw_exit_flags
+{
+    SW_EXIT_IN_COROUTINE = 1 << 1,
+    SW_EXIT_IN_SERVER = 1<< 2
+};
+
+static int coro_exit_handler(zend_execute_data *execute_data)
+{
+    zval ex;
+    zend_object *obj;
+    zend_long flags = 0;
+    if (sw_get_current_cid() != -1)
+    {
+        flags |= SW_EXIT_IN_COROUTINE;
+    }
+    if (SwooleG.serv && SwooleG.serv->gs->start)
+    {
+        flags |= SW_EXIT_IN_SERVER;
+    }
+    if (flags)
+    {
+        const zend_op *opline = EX(opline);
+        zval _exit_status;
+        zval *exit_status = NULL;
+
+        if (opline->op1_type != IS_UNUSED)
+        {
+            if (opline->op1_type == IS_CONST)
+            {
+                // see: https://github.com/php/php-src/commit/e70618aff6f447a298605d07648f2ce9e5a284f5
+#ifdef EX_CONSTANT
+                exit_status = EX_CONSTANT(opline->op1);
+#else
+                exit_status = RT_CONSTANT(opline, opline->op1);
+#endif
+            }
+            else
+            {
+                exit_status = EX_VAR(opline->op1.var);
+            }
+            if (Z_ISREF_P(exit_status))
+            {
+                exit_status = Z_REFVAL_P(exit_status);
+            }
+        }
+        else
+        {
+            exit_status = &_exit_status;
+            ZVAL_NULL(exit_status);
+        }
+        obj = zend_throw_error_exception(swoole_exit_exception_class_entry_ptr, "swoole exit.", 0, E_ERROR TSRMLS_CC);
+        ZVAL_OBJ(&ex, obj);
+        zend_update_property_long(swoole_exit_exception_class_entry_ptr, &ex, ZEND_STRL("flags"), flags);
+        Z_TRY_ADDREF_P(exit_status);
+        zend_update_property(swoole_exit_exception_class_entry_ptr, &ex, ZEND_STRL("status"), exit_status);
+    }
+
+    return ZEND_USER_OPCODE_DISPATCH;
+}
+
 void swoole_coroutine_util_init(int module_number TSRMLS_DC)
 {
     SWOOLE_INIT_CLASS_ENTRY(swoole_coroutine_util_ce, "swoole_coroutine", "Swoole\\Coroutine", swoole_coroutine_util_methods);
     swoole_coroutine_util_class_entry_ptr = zend_register_internal_class(&swoole_coroutine_util_ce TSRMLS_CC);
 
+    INIT_CLASS_ENTRY(swoole_coroutine_iterator_ce, "Swoole\\Coroutine\\Iterator", iterator_methods);
+    swoole_coroutine_iterator_class_entry_ptr = zend_register_internal_class(&swoole_coroutine_iterator_ce TSRMLS_CC);
+    zend_class_implements(swoole_coroutine_iterator_class_entry_ptr TSRMLS_CC, 1, zend_ce_iterator);
+#ifdef SW_HAVE_COUNTABLE
+    zend_class_implements(swoole_coroutine_iterator_class_entry_ptr TSRMLS_CC, 1, zend_ce_countable);
+#endif
+
     if (SWOOLE_G(use_namespace))
     {
         sw_zend_register_class_alias("swoole_coroutine", swoole_coroutine_util_class_entry_ptr);
@@ -151,6 +273,24 @@ void swoole_coroutine_util_init(int module_number TSRMLS_DC)
         sw_zend_register_class_alias("Co", swoole_coroutine_util_class_entry_ptr);
     }
     defer_coros = swHashMap_new(SW_HASHMAP_INIT_BUCKET_N, NULL);
+
+    //prohibit exit in coroutine
+    INIT_CLASS_ENTRY(swoole_exit_exception_ce, "Swoole\\ExitException", swoole_exit_exception_methods);
+    swoole_exit_exception_class_entry_ptr = zend_register_internal_class_ex(&swoole_exit_exception_ce, zend_exception_get_default());
+    SWOOLE_DEFINE(EXIT_IN_COROUTINE);
+    SWOOLE_DEFINE(EXIT_IN_SERVER);
+    ori_exit_handler = zend_get_user_opcode_handler(ZEND_EXIT);
+    zend_set_user_opcode_handler(ZEND_EXIT, coro_exit_handler);
+}
+
+static PHP_METHOD(swoole_exit_exception, getFlags)
+{
+    RETURN_LONG(Z_LVAL_P(sw_zend_read_property(Z_OBJCE_P(getThis()), getThis(), ZEND_STRL("flags"), 1)));
+}
+
+static PHP_METHOD(swoole_exit_exception, getStatus)
+{
+    RETURN_ZVAL(sw_zend_read_property(Z_OBJCE_P(getThis()), getThis(), ZEND_STRL("status"), 1), 0, 0);
 }
 
 /*
@@ -340,6 +480,7 @@ static PHP_METHOD(swoole_coroutine_util, stats)
 {
     array_init(return_value);
     sw_add_assoc_long_ex(return_value, ZEND_STRS("coroutine_num"), COROG.coro_num);
+    sw_add_assoc_long_ex(return_value, ZEND_STRS("coroutine_peak_num"), COROG.peak_coro_num);
 }
 
 static PHP_METHOD(swoole_coroutine_util, getuid)
@@ -782,7 +923,7 @@ static PHP_METHOD(swoole_coroutine_util, fread)
         swAio_init();
     }
 
-    swTrace("fd=%d, offset=%ld, length=%ld", fd, ev.offset, ev.nbytes);
+    swTrace("fd=%d, offset=%jd, length=%ld", fd, (intmax_t) ev.offset, ev.nbytes);
 
     int ret = swAio_dispatch(&ev);
     if (ret < 0)
@@ -863,7 +1004,7 @@ static PHP_METHOD(swoole_coroutine_util, fgets)
         swAio_init();
     }
 
-    swTrace("fd=%d, offset=%ld, length=%ld", fd, ev.offset, ev.nbytes);
+    swTrace("fd=%d, offset=%jd, length=%ld", fd, (intmax_t) ev.offset, ev.nbytes);
 
     int ret = swAio_dispatch(&ev);
     if (ret < 0)
@@ -949,7 +1090,7 @@ static PHP_METHOD(swoole_coroutine_util, fwrite)
 
     php_swoole_check_aio();
 
-    swTrace("fd=%d, offset=%ld, length=%ld", fd, ev.offset, ev.nbytes);
+    swTrace("fd=%d, offset=%jd, length=%ld", fd, (intmax_t) ev.offset, ev.nbytes);
 
     int ret = swAio_dispatch(&ev);
     if (ret < 0)
@@ -1313,4 +1454,93 @@ static PHP_METHOD(swoole_coroutine_util, getaddrinfo)
     coro_yield();
 }
 
-#endif
+static PHP_METHOD(swoole_coroutine_util, getBackTrace)
+{
+    zend_long cid;
+    zend_long options = DEBUG_BACKTRACE_PROVIDE_OBJECT;
+    zend_long limit = 0;
+
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l|ll", &cid) == FAILURE)
+    {
+        return;
+    }
+    if (cid == sw_get_current_cid())
+    {
+        zend_fetch_debug_backtrace(return_value, 0, options, limit);
+    }
+    else
+    {
+        coro_task *task = (coro_task *) coroutine_get_ptr_by_cid(cid);
+        if (task == NULL)
+        {
+            RETURN_FALSE;
+        }
+        zend_execute_data *ex_backup = EG(current_execute_data);
+        EG(current_execute_data) = task->yield_execute_data;
+        zend_fetch_debug_backtrace(return_value, 0, options, limit);
+        EG(current_execute_data) = ex_backup;
+    }
+}
+
+static PHP_METHOD(swoole_coroutine_iterator, rewind)
+{
+    coroutine_iterator *itearator = swoole_get_object(getThis());
+    bzero(itearator, sizeof(coroutine_iterator));
+    itearator->count = COROG.coro_num;
+}
+
+static PHP_METHOD(swoole_coroutine_iterator, valid)
+{
+    coroutine_iterator *itearator = swoole_get_object(getThis());
+    int cid = itearator->current_cid;
+
+    for (; itearator->count > 0 && cid < MAX_CORO_NUM_LIMIT + 1; cid++)
+    {
+        if (coroutine_get_by_id(cid))
+        {
+            itearator->current_cid = cid;
+            itearator->index++;
+            itearator->count--;
+            RETURN_TRUE;
+        }
+    }
+    RETURN_FALSE;
+}
+
+static PHP_METHOD(swoole_coroutine_iterator, current)
+{
+    coroutine_iterator *itearator = swoole_get_object(getThis());
+    RETURN_LONG(itearator->current_cid);
+}
+
+static PHP_METHOD(swoole_coroutine_iterator, next)
+{
+    coroutine_iterator *itearator = swoole_get_object(getThis());
+    itearator->current_cid++;
+}
+
+PHP_METHOD(swoole_coroutine_iterator, key)
+{
+    coroutine_iterator *itearator = swoole_get_object(getThis());
+    RETURN_LONG(itearator->index);
+}
+
+static PHP_METHOD(swoole_coroutine_iterator, count)
+{
+    RETURN_LONG(COROG.coro_num);
+}
+
+static PHP_METHOD(swoole_coroutine_iterator, __destruct)
+{
+    coroutine_iterator *i = swoole_get_object(getThis());
+    efree(i);
+    swoole_set_object(getThis(), NULL);
+}
+
+static PHP_METHOD(swoole_coroutine_util, listCoroutines)
+{
+    object_init_ex(return_value, swoole_coroutine_iterator_class_entry_ptr);
+    coroutine_iterator *i = emalloc(sizeof(coroutine_iterator));
+    bzero(i, sizeof(coroutine_iterator));
+    swoole_set_object(return_value, i);
+}
