@@ -23,14 +23,12 @@
 
 typedef struct
 {
-#if PHP_MAJOR_VERSION >= 7
     struct
     {
         zval cb_read;
         zval cb_write;
         zval socket;
     } stack;
-#endif
     zval *cb_read;
     zval *cb_write;
     zval *socket;
@@ -38,9 +36,7 @@ typedef struct
 
 typedef struct
 {
-#if PHP_MAJOR_VERSION >= 7
     zval _callback;
-#endif
     zval *callback;
 } php_defer_callback;
 
@@ -84,7 +80,6 @@ static int php_swoole_event_onRead(swReactor *reactor, swEvent *event)
     zval **args[1];
     php_reactor_fd *fd = event->socket->object;
 
-    SWOOLE_GET_TSRMLS;
 
     args[0] = &fd->socket;
 
@@ -111,7 +106,6 @@ static int php_swoole_event_onWrite(swReactor *reactor, swEvent *event)
     zval **args[1];
     php_reactor_fd *fd = event->socket->object;
 
-    SWOOLE_GET_TSRMLS;
 
     if (!fd->cb_write)
     {
@@ -139,7 +133,6 @@ static int php_swoole_event_onWrite(swReactor *reactor, swEvent *event)
 
 static int php_swoole_event_onError(swReactor *reactor, swEvent *event)
 {
-    SWOOLE_GET_TSRMLS;
 
     int error;
     socklen_t len = sizeof(error);
@@ -166,7 +159,6 @@ static void php_swoole_event_onDefer(void *_cb)
 {
     php_defer_callback *defer = _cb;
 
-    SWOOLE_GET_TSRMLS;
     zval *retval;
     if (sw_call_user_function_ex(EG(function_table), NULL, defer->callback, &retval, 0, NULL, 0, NULL TSRMLS_CC) == FAILURE)
     {
@@ -189,7 +181,6 @@ static void php_swoole_event_onEndCallback(void *_cb)
 {
     php_defer_callback *defer = _cb;
 
-    SWOOLE_GET_TSRMLS;
     zval *retval;
     if (sw_call_user_function_ex(EG(function_table), NULL, defer->callback, &retval, 0, NULL, 0, NULL TSRMLS_CC) == FAILURE)
     {
@@ -206,20 +197,65 @@ static void php_swoole_event_onEndCallback(void *_cb)
     }
 }
 
-void php_swoole_event_init(void)
+void php_swoole_check_reactor()
 {
+    if (likely(SwooleWG.reactor_init))
+    {
+        return;
+    }
+
+    if (!SWOOLE_G(cli))
+    {
+        swoole_php_fatal_error(E_ERROR, "async-io must be used in PHP CLI mode.");
+        return;
+    }
+
+    if (swIsTaskWorker())
+    {
+        swoole_php_fatal_error(E_ERROR, "can't use async-io in task process.");
+        return;
+    }
+
+    if (SwooleG.main_reactor == NULL)
+    {
+        swTraceLog(SW_TRACE_PHP, "init reactor");
+
+        SwooleG.main_reactor = (swReactor *) sw_malloc(sizeof(swReactor));
+        if (SwooleG.main_reactor == NULL)
+        {
+            swoole_php_fatal_error(E_ERROR, "malloc failed.");
+            return;
+        }
+        if (swReactor_create(SwooleG.main_reactor, SW_REACTOR_MAXEVENTS) < 0)
+        {
+            swoole_php_fatal_error(E_ERROR, "failed to create reactor.");
+            return;
+        }
+
+#ifdef SW_COROUTINE
+        SwooleG.main_reactor->can_exit = php_coroutine_reactor_can_exit;
+#endif
+
+        //client, swoole_event_exit will set swoole_running = 0
+        SwooleWG.in_client = 1;
+        SwooleWG.reactor_wait_onexit = 1;
+        SwooleWG.reactor_ready = 0;
+        //only client side
+        php_swoole_at_shutdown("swoole_event_wait");
+    }
+
     SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_USER | SW_EVENT_READ, php_swoole_event_onRead);
     SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_USER | SW_EVENT_WRITE, php_swoole_event_onWrite);
     SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_USER | SW_EVENT_ERROR, php_swoole_event_onError);
     SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_WRITE, swReactor_onWrite);
+
+    SwooleWG.reactor_init = 1;
 }
 
 void php_swoole_event_wait()
 {
-    SWOOLE_GET_TSRMLS;
     if (SwooleWG.in_client == 1 && SwooleWG.reactor_ready == 0 && SwooleG.running)
     {
-#if PHP_MAJOR_VERSION >= 7
         if (PG(last_error_message))
         {
             switch (PG(last_error_type))
@@ -233,7 +269,6 @@ void php_swoole_event_wait()
                 break;
             }
         }
-#endif
         SwooleWG.reactor_ready = 1;
 
 #ifdef HAVE_SIGNALFD
@@ -371,7 +406,6 @@ int swoole_convert_to_fd_ex(zval *zfd, int *async TSRMLS_DC)
 #ifdef SWOOLE_SOCKETS_SUPPORT
 php_socket* swoole_convert_to_socket(int sock)
 {
-    SWOOLE_GET_TSRMLS;
     php_socket *socket_object = emalloc(sizeof *socket_object);
     bzero(socket_object, sizeof(php_socket));
     socket_object->bsd_socket = sock;
@@ -686,20 +720,9 @@ PHP_FUNCTION(swoole_event_defer)
 
     php_swoole_check_reactor();
 
-    //the event loop is not started
-    if (SwooleG.main_reactor->start == 0)
-    {
-        SW_CHECK_RETURN (php_swoole_add_timer(1, callback, NULL, 0 TSRMLS_CC));
-    }
-
     php_defer_callback *defer = emalloc(sizeof(php_defer_callback));
-
-#if PHP_MAJOR_VERSION >= 7
     defer->callback = &defer->_callback;
     memcpy(defer->callback, callback, sizeof(zval));
-#else
-    defer->callback = callback;
-#endif
     sw_zval_add_ref(&callback);
     SW_CHECK_RETURN(SwooleG.main_reactor->defer(SwooleG.main_reactor, php_swoole_event_onDefer, defer));
 }
@@ -746,12 +769,8 @@ PHP_FUNCTION(swoole_event_cycle)
 
     php_defer_callback *cb = emalloc(sizeof(php_defer_callback));
 
-#if PHP_MAJOR_VERSION >= 7
     cb->callback = &cb->_callback;
     memcpy(cb->callback, callback, sizeof(zval));
-#else
-    cb->callback = callback;
-#endif
     sw_zval_add_ref(&callback);
 
     if (before == 0)
