@@ -33,13 +33,17 @@
 #include <main/php_variables.h>
 
 #include "websocket.h"
-#include "Connection.h"
+#include "connection.h"
 #include "base64.h"
 
 #ifdef SW_HAVE_ZLIB
 #include <zlib.h>
 #endif
 
+#ifdef SW_HAVE_BROTLI
+#include <brotli/encode.h>
+#endif
+
 #ifdef SW_USE_HTTP2
 #include "http2.h"
 #endif
@@ -48,8 +52,6 @@
 #include "thirdparty/picohttpparser/picohttpparser.h"
 #endif
 
-static swArray *http_client_array;
-
 swString *swoole_http_buffer;
 #ifdef SW_HAVE_ZLIB
 swString *swoole_zlib_buffer;
@@ -262,9 +264,7 @@ static PHP_METHOD(swoole_http_response, header);
 static PHP_METHOD(swoole_http_response, initHeader);
 static PHP_METHOD(swoole_http_response, detach);
 static PHP_METHOD(swoole_http_response, create);
-#ifdef SW_HAVE_ZLIB
 static PHP_METHOD(swoole_http_response, gzip);
-#endif
 #ifdef SW_USE_HTTP2
 static PHP_METHOD(swoole_http_response, trailer);
 #endif
@@ -340,14 +340,13 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_http_server_on, 0, 0, 2)
     ZEND_ARG_INFO(0, callback)
 ZEND_END_ARG_INFO()
 
-#ifdef SW_HAVE_ZLIB
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_http_response_gzip, 0, 0, 0)
     ZEND_ARG_INFO(0, compress_level)
 ZEND_END_ARG_INFO()
-#endif
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_http_response_status, 0, 0, 1)
     ZEND_ARG_INFO(0, http_code)
+    ZEND_ARG_INFO(0, reason)
 ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_http_response_header, 0, 0, 2)
@@ -443,9 +442,7 @@ const zend_function_entry swoole_http_response_methods[] =
     PHP_ME(swoole_http_response, cookie, arginfo_swoole_http_response_cookie, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_response, rawcookie, arginfo_swoole_http_response_cookie, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_response, status, arginfo_swoole_http_response_status, ZEND_ACC_PUBLIC)
-#ifdef SW_HAVE_ZLIB
     PHP_ME(swoole_http_response, gzip, arginfo_swoole_http_response_gzip, ZEND_ACC_PUBLIC)
-#endif
     PHP_ME(swoole_http_response, header, arginfo_swoole_http_response_header, ZEND_ACC_PUBLIC)
 #ifdef SW_USE_HTTP2
     PHP_ME(swoole_http_response, trailer, arginfo_swoole_http_response_trailer, ZEND_ACC_PUBLIC)
@@ -670,6 +667,12 @@ static int http_request_on_header_value(php_http_parser *parser, const char *at,
             }
         }
     }
+#ifdef SW_HAVE_ZLIB
+    else if (SwooleG.serv->http_compression && strncmp(header_name, "accept-encoding", header_len) == 0)
+    {
+        swoole_http_get_compression_method(ctx, at, length);
+    }
+#endif
 
     zval *header = ctx->request.zheader;
     sw_add_assoc_stringl_ex(header, header_name, ctx->current_header_name_len + 1, (char *) at, length, 1);
@@ -715,7 +718,7 @@ static int multipart_body_on_header_value(multipart_parser* p, const char *at, s
      */
     if (ctx->input_var_num > PG(max_input_vars))
     {
-        swoole_php_error(E_WARNING, "Input variables exceeded %ld. "
+        swoole_php_error(E_WARNING, "Input variables exceeded " ZEND_LONG_FMT ". "
                 "To increase the limit change max_input_vars in php.ini.", PG(max_input_vars));
         return SW_OK;
     }
@@ -1031,40 +1034,36 @@ static int http_onReceive(swServer *serv, swEventData *req)
     }
 
     int fd = req->info.fd;
+    int from_fd = req->info.from_fd;
+
     swConnection *conn = swServer_connection_verify_no_ssl(SwooleG.serv, fd);
     if (!conn)
     {
         swoole_error_log(SW_LOG_NOTICE, SW_ERROR_SESSION_NOT_EXIST, "connection[%d] is closed.", fd);
         return SW_ERR;
     }
-    swListenPort *port = serv->connection_list[req->info.from_fd].object;
+
+    swListenPort *port = serv->connection_list[from_fd].object;
     //other server port
     if (!port->open_http_protocol)
     {
         return php_swoole_onReceive(serv, req);
     }
+
     //websocket client
     if (conn->websocket_status == WEBSOCKET_STATUS_ACTIVE)
     {
         return swoole_websocket_onMessage(req);
     }
 
-    swoole_http_client *client = swArray_alloc(http_client_array, conn->fd);
-    if (!client)
-    {
-        return SW_OK;
-    }
-    client->fd = fd;
-
 #ifdef SW_USE_HTTP2
     if (conn->http2_stream)
     {
-        client->http2 = 1;
-        return swoole_http2_onFrame(client, req);
+        return swoole_http2_onFrame(conn, req);
     }
 #endif
 
-    http_context *ctx = swoole_http_context_new(client TSRMLS_CC);
+    http_context *ctx = swoole_http_context_new(fd TSRMLS_CC);
     php_http_parser *parser = &ctx->parser;
     zval *zserver = ctx->request.zserver;
 
@@ -1086,7 +1085,6 @@ static int http_onReceive(swServer *serv, swEventData *req)
     if (n < 0)
     {
         sw_zval_free(zdata);
-        bzero(client, sizeof(swoole_http_client));
         swWarn("php_http_parser_execute failed.");
         if (conn->websocket_status == WEBSOCKET_STATUS_CONNECTION)
         {
@@ -1125,11 +1123,10 @@ static int http_onReceive(swServer *serv, swEventData *req)
 
         swoole_set_property(zrequest_object, 0, zdata);
 
-        add_assoc_long(ctx->request.zserver, "server_port", swConnection_get_port(&SwooleG.serv->connection_list[conn->from_fd]));
-        add_assoc_long(ctx->request.zserver, "remote_port", swConnection_get_port(conn));
+        add_assoc_long(zserver, "server_port", swConnection_get_port(&SwooleG.serv->connection_list[conn->from_fd]));
+        add_assoc_long(zserver, "remote_port", swConnection_get_port(conn));
         sw_add_assoc_string(zserver, "remote_addr", swConnection_get_ip(conn), 1);
         add_assoc_long(zserver, "master_time", conn->last_time);
-
         if (ctx->request.version == 101)
         {
             sw_add_assoc_string(zserver, "server_protocol", "HTTP/1.1", 1);
@@ -1138,33 +1135,36 @@ static int http_onReceive(swServer *serv, swEventData *req)
         {
             sw_add_assoc_string(zserver, "server_protocol", "HTTP/1.0", 1);
         }
-
         sw_add_assoc_string(zserver, "server_software", SW_HTTP_SERVER_SOFTWARE, 1);
 
-        zval *zcallback = php_swoole_server_get_callback(serv, req->info.from_fd, SW_SERVER_CB_onHandShake);
-        //websocket handshake
-        if (conn->websocket_status == WEBSOCKET_STATUS_CONNECTION && zcallback == NULL)
-        {
-            swoole_websocket_onHandshake(port, ctx);
-            goto free_object;
-        }
-
         int callback_type = 0;
+        zval *zcallback = NULL;
+
         if (conn->websocket_status == WEBSOCKET_STATUS_CONNECTION)
         {
-            callback_type = SW_SERVER_CB_onHandShake;
-            conn->websocket_status = WEBSOCKET_STATUS_HANDSHAKE;
-            ctx->upgrade = 1;
+            //websocket handshake
+            zcallback = php_swoole_server_get_callback(serv, from_fd, SW_SERVER_CB_onHandShake);
+            if (zcallback == NULL)
+            {
+                swoole_websocket_onHandshake(port, ctx);
+                goto _free_object;
+            }
+            else
+            {
+                callback_type = SW_SERVER_CB_onHandShake;
+                conn->websocket_status = WEBSOCKET_STATUS_HANDSHAKE;
+                ctx->upgrade = 1;
+            }
         }
         else
         {
             callback_type = SW_SERVER_CB_onRequest;
-            zcallback = php_swoole_server_get_callback(serv, req->info.from_fd, SW_SERVER_CB_onRequest);
+            zcallback = php_swoole_server_get_callback(serv, from_fd, SW_SERVER_CB_onRequest);
             //no have onRequest callback
             if (zcallback == NULL)
             {
                 swoole_websocket_onRequest(ctx);
-                goto free_object;
+                goto _free_object;
             }
         }
 
@@ -1174,17 +1174,15 @@ static int http_onReceive(swServer *serv, swEventData *req)
             args[0] = zrequest_object;
             args[1] = zresponse_object;
 
-            zend_fcall_info_cache *cache = php_swoole_server_get_cache(serv, req->info.from_fd, callback_type);
+            zend_fcall_info_cache *cache = php_swoole_server_get_cache(serv, from_fd, callback_type);
             int ret = coro_create(cache, args, 2, &retval, NULL, NULL);
             if (ret < 0)
             {
-                sw_zval_ptr_dtor(&zrequest_object);
-                sw_zval_ptr_dtor(&zresponse_object);
                 if (ret == CORO_LIMIT)
                 {
                     serv->factory.end(&SwooleG.serv->factory, fd);
                 }
-                return SW_OK;
+                goto _free_object;
             }
         }
         else
@@ -1193,8 +1191,7 @@ static int http_onReceive(swServer *serv, swEventData *req)
             args[0] = &zrequest_object;
             args[1] = &zresponse_object;
 
-            zcallback = php_swoole_server_get_callback(serv, req->info.from_fd, callback_type);
-            zend_fcall_info_cache *fci_cache = php_swoole_server_get_cache(serv, req->info.from_fd, callback_type);
+            zend_fcall_info_cache *fci_cache = php_swoole_server_get_cache(serv, from_fd, callback_type);
             if (sw_call_user_function_fast(zcallback, fci_cache, &retval, 2, args TSRMLS_CC) == FAILURE)
             {
                 swoole_php_error(E_WARNING, "onRequest handler error");
@@ -1214,7 +1211,8 @@ static int http_onReceive(swServer *serv, swEventData *req)
                 conn->websocket_status = WEBSOCKET_STATUS_ACTIVE;
             }
         }
-        free_object: bzero(client, sizeof(swoole_http_client));
+
+        _free_object:
         sw_zval_ptr_dtor(&zrequest_object);
         sw_zval_ptr_dtor(&zresponse_object);
         if (retval)
@@ -1229,28 +1227,17 @@ static int http_onReceive(swServer *serv, swEventData *req)
 static void http_onClose(swServer *serv, swDataHead *ev)
 {
     int fd = ev->fd;
-
     swConnection *conn = swWorker_get_connection(SwooleG.serv, fd);
     if (!conn)
     {
         return;
     }
-    swoole_http_client *client = swArray_fetch(http_client_array, conn->fd);
-    if (!client)
-    {
-        return;
-    }
 #ifdef SW_USE_HTTP2
-    if (client->http2)
+    if (conn->http2_stream)
     {
-        swoole_http2_free(client);
+        swoole_http2_free(conn);
     }
 #endif
-    zval *zcallback = php_swoole_server_get_callback(serv, ev->from_fd, SW_SERVER_CB_onClose);
-    if (!zcallback)
-    {
-        return;
-    }
     php_swoole_onClose(serv, ev);
 }
 
@@ -1290,7 +1277,10 @@ void swoole_http_server_init(int module_number TSRMLS_DC)
         sw_zend_register_class_alias("Co\\Http\\Response", swoole_http_response_class_entry_ptr);
     }
 
-    zend_declare_property_long(swoole_http_request_class_entry_ptr, SW_STRL("fd")-1, 0,  ZEND_ACC_PUBLIC TSRMLS_CC);
+    zend_declare_property_long(swoole_http_request_class_entry_ptr, SW_STRL("fd")-1, 0, ZEND_ACC_PUBLIC TSRMLS_CC);
+#ifdef SW_USE_HTTP2
+    zend_declare_property_long(swoole_http_request_class_entry_ptr, SW_STRL("streamId")-1, 0, ZEND_ACC_PUBLIC TSRMLS_CC);
+#endif
     zend_declare_property_null(swoole_http_request_class_entry_ptr, SW_STRL("header")-1, ZEND_ACC_PUBLIC TSRMLS_CC);
     zend_declare_property_null(swoole_http_request_class_entry_ptr, SW_STRL("server")-1, ZEND_ACC_PUBLIC TSRMLS_CC);
     zend_declare_property_null(swoole_http_request_class_entry_ptr, SW_STRL("request")-1, ZEND_ACC_PUBLIC TSRMLS_CC);
@@ -1371,7 +1361,7 @@ static PHP_METHOD(swoole_http_server, on)
     }
 }
 
-http_context* swoole_http_context_new(swoole_http_client* client TSRMLS_DC)
+http_context* swoole_http_context_new(int fd)
 {
     http_context *ctx = emalloc(sizeof(http_context));
     if (!ctx)
@@ -1382,28 +1372,20 @@ http_context* swoole_http_context_new(swoole_http_client* client TSRMLS_DC)
     bzero(ctx, sizeof(http_context));
 
     zval *zrequest_object;
-#if PHP_MAJOR_VERSION >= 7
     zrequest_object = &ctx->request._zobject;
-#else
-    SW_ALLOC_INIT_ZVAL(zrequest_object);
-#endif
     ctx->request.zobject = zrequest_object;
     object_init_ex(zrequest_object, swoole_http_request_class_entry_ptr);
     swoole_set_object(zrequest_object, ctx);
 
     zval *zresponse_object;
-#if PHP_MAJOR_VERSION >= 7
     zresponse_object = &ctx->response._zobject;
-#else
-    SW_ALLOC_INIT_ZVAL(zresponse_object);
-#endif
     ctx->response.zobject = zresponse_object;
     object_init_ex(zresponse_object, swoole_http_response_class_entry_ptr);
     swoole_set_object(zresponse_object, ctx);
 
     //socket fd
-    zend_update_property_long(swoole_http_response_class_entry_ptr, zresponse_object, ZEND_STRL("fd"), client->fd TSRMLS_CC);
-    zend_update_property_long(swoole_http_request_class_entry_ptr, zrequest_object, ZEND_STRL("fd"), client->fd TSRMLS_CC);
+    zend_update_property_long(swoole_http_response_class_entry_ptr, zresponse_object, ZEND_STRL("fd"), fd TSRMLS_CC);
+    zend_update_property_long(swoole_http_request_class_entry_ptr, zrequest_object, ZEND_STRL("fd"), fd TSRMLS_CC);
 
 #if PHP_MEMORY_DEBUG
     php_vmstat.new_http_request ++;
@@ -1415,8 +1397,7 @@ http_context* swoole_http_context_new(swoole_http_client* client TSRMLS_DC)
     zval *zserver;
     swoole_http_server_array_init(server, request);
 
-    ctx->fd = client->fd;
-    ctx->client = client;
+    ctx->fd = fd;
 
     return ctx;
 }
@@ -1425,6 +1406,7 @@ void swoole_http_context_free(http_context *ctx TSRMLS_DC)
 {
     swoole_set_object(ctx->response.zobject, NULL);
     http_request *req = &ctx->request;
+    http_response *res = &ctx->response;
     if (req->path)
     {
         efree(req->path);
@@ -1435,6 +1417,10 @@ void swoole_http_context_free(http_context *ctx TSRMLS_DC)
         swString_free(req->post_buffer);
     }
 #endif
+    if (res->reason)
+    {
+        efree(res->reason);
+    }
     efree(ctx);
 }
 
@@ -1592,13 +1578,6 @@ static PHP_METHOD(swoole_http_server, start)
         RETURN_FALSE;
     }
 
-    http_client_array = swArray_new(1024, sizeof(swoole_http_client));
-    if (!http_client_array)
-    {
-        swoole_php_fatal_error(E_ERROR, "swArray_new(1024, %ld) failed.", sizeof(swoole_http_client));
-        RETURN_FALSE;
-    }
-
     swoole_http_buffer = swString_new(SW_HTTP_RESPONSE_INIT_SIZE);
     if (!swoole_http_buffer)
     {
@@ -1755,11 +1734,17 @@ static PHP_METHOD(swoole_http_response, write)
         return;
     }
 
-    if (ctx->http2)
+#ifdef SW_USE_HTTP2
+    if (ctx->stream)
     {
         swoole_php_error(E_WARNING, "Http2 client does not support HTTP-CHUNK.");
         RETURN_FALSE;
     }
+#endif
+
+#ifdef SW_HAVE_ZLIB
+    ctx->enable_compression = 0;
+#endif
 
     if (!ctx->send_header)
     {
@@ -1791,38 +1776,20 @@ static PHP_METHOD(swoole_http_response, write)
         http_body.length = length;
     }
 
+    // Why not enable compression?
+    // If both compression and chunked encoding are enabled,
+    // then the content stream is first compressed, then chunked;
+    // so the chunk encoding itself is not compressed,
+    // **and the data in each chunk is not compressed individually.**
+    // The remote endpoint then decodes the stream by concatenating the chunks and uncompressing the result.
     swString_clear(swoole_http_buffer);
-
-    char *hex_string;
-    int hex_len;
-
-#ifdef SW_HAVE_ZLIB
-    if (ctx->gzip_enable)
-    {
-        swoole_http_response_compress(&http_body, ctx->gzip_level);
-
-        hex_string = swoole_dec2hex(swoole_zlib_buffer->length, 16);
-        hex_len = strlen(hex_string);
-
-        //"%*s\r\n%*s\r\n", hex_len, hex_string, body.length, body.str
-        swString_append_ptr(swoole_http_buffer, hex_string, hex_len);
-        swString_append_ptr(swoole_http_buffer, SW_STRL("\r\n") - 1);
-        swString_append(swoole_http_buffer, swoole_zlib_buffer);
-        swString_append_ptr(swoole_http_buffer, SW_STRL("\r\n") - 1);
-    }
-    else
-#endif
-    {
-        hex_string = swoole_dec2hex(http_body.length, 16);
-        hex_len = strlen(hex_string);
-
-        //"%*s\r\n%*s\r\n", hex_len, hex_string, body.length, body.str
-        swString_append_ptr(swoole_http_buffer, hex_string, hex_len);
-        swString_append_ptr(swoole_http_buffer, SW_STRL("\r\n") - 1);
-        swString_append_ptr(swoole_http_buffer, http_body.str, http_body.length);
-        swString_append_ptr(swoole_http_buffer, SW_STRL("\r\n") - 1);
-    }
-
+    char *hex_string = swoole_dec2hex(http_body.length, 16);
+    int hex_len = strlen(hex_string);
+    //"%*s\r\n%*s\r\n", hex_len, hex_string, body.length, body.str
+    swString_append_ptr(swoole_http_buffer, hex_string, hex_len);
+    swString_append_ptr(swoole_http_buffer, SW_STRL("\r\n") - 1);
+    swString_append_ptr(swoole_http_buffer, http_body.str, http_body.length);
+    swString_append_ptr(swoole_http_buffer, SW_STRL("\r\n") - 1);
     sw_free(hex_string);
 
     swServer *serv = SwooleG.serv;
@@ -1867,23 +1834,31 @@ static void http_build_header(http_context *ctx, zval *object, swString *respons
 
     swServer *serv = SwooleG.serv;
 
-    char buf[SW_HTTP_HEADER_MAX_SIZE];
+    char *buf = SwooleTG.buffer_stack->str;
+    size_t l_buf = SwooleTG.buffer_stack->size;
     int n;
     char *date_str;
 
     /**
      * http status line
      */
-    n = snprintf(buf, sizeof(buf), "HTTP/1.1 %s\r\n", http_status_message(ctx->response.status));
+    if (!ctx->response.reason)
+    {
+        n = snprintf(buf, l_buf, "HTTP/1.1 %s\r\n", http_status_message(ctx->response.status));
+    }
+    else
+    {
+        n = snprintf(buf, l_buf, "HTTP/1.1 %d %s\r\n", ctx->response.status, ctx->response.reason);
+    }
     swString_append_ptr(response, buf, n);
 
     /**
      * http header
      */
     zval *header = ctx->response.zheader;
+    int flag = 0x0;
     if (header)
     {
-        int flag = 0x0;
         HashTable *ht = Z_ARRVAL_P(header);
         zval *value = NULL;
         char *key = NULL;
@@ -1912,50 +1887,30 @@ static void http_build_header(http_context *ctx, zval *object, swString *respons
             {
                 flag |= HTTP_RESPONSE_CONTENT_TYPE;
             }
-            n = snprintf(buf, sizeof(buf), "%*s: %*s\r\n", keylen - 1, key, (int)Z_STRLEN_P(value), Z_STRVAL_P(value));
+            else if (strncasecmp(key, "Transfer-Encoding", keylen) == 0)
+            {
+                flag |= HTTP_RESPONSE_TRANSFER_ENCODING;
+            }
+            n = snprintf(buf, l_buf, "%*s: %*s\r\n", keylen - 1, key, (int)Z_STRLEN_P(value), Z_STRVAL_P(value));
             swString_append_ptr(response, buf, n);
         }
         SW_HASHTABLE_FOREACH_END();
         (void)type;
-
-        if (!(flag & HTTP_RESPONSE_SERVER))
-        {
-            swString_append_ptr(response, ZEND_STRL("Server: "SW_HTTP_SERVER_SOFTWARE"\r\n"));
-        }
-        //websocket protocol
-        if (ctx->upgrade == 1)
-        {
-            swString_append_ptr(response, ZEND_STRL("\r\n"));
-            ctx->send_header = 1;
-            return;
-        }
-        if (!(flag & HTTP_RESPONSE_CONNECTION))
-        {
-            if (ctx->keepalive)
-            {
-                swString_append_ptr(response, ZEND_STRL("Connection: keep-alive\r\n"));
-            }
-            else
-            {
-                swString_append_ptr(response, ZEND_STRL("Connection: close\r\n"));
-            }
-        }
-        if (!(flag & HTTP_RESPONSE_CONTENT_TYPE))
-        {
-            swString_append_ptr(response, ZEND_STRL("Content-Type: text/html\r\n"));
-        }
-        if (!(flag & HTTP_RESPONSE_DATE))
-        {
-            date_str = sw_php_format_date(ZEND_STRL(SW_HTTP_DATE_FORMAT), serv->gs->now, 0 TSRMLS_CC);
-            n = snprintf(buf, sizeof(buf), "Date: %s\r\n", date_str);
-            swString_append_ptr(response, buf, n);
-            efree(date_str);
-        }
     }
-    else
+
+    if (!(flag & HTTP_RESPONSE_SERVER))
+    {
+        swString_append_ptr(response, ZEND_STRL("Server: "SW_HTTP_SERVER_SOFTWARE"\r\n"));
+    }
+    //websocket protocol
+    if (ctx->upgrade == 1)
+    {
+        swString_append_ptr(response, ZEND_STRL("\r\n"));
+        ctx->send_header = 1;
+        return;
+    }
+    if (!(flag & HTTP_RESPONSE_CONNECTION))
     {
-        //Server
-        swString_append_ptr(response, ZEND_STRL("Server: "SW_HTTP_SERVER_SOFTWARE"\r\nContent-Type: text/html\r\n"));
         if (ctx->keepalive)
         {
             swString_append_ptr(response, ZEND_STRL("Connection: keep-alive\r\n"));
@@ -1964,33 +1919,39 @@ static void http_build_header(http_context *ctx, zval *object, swString *respons
         {
             swString_append_ptr(response, ZEND_STRL("Connection: close\r\n"));
         }
-        //Date
-        date_str = sw_php_format_date(ZEND_STRL(SW_HTTP_DATE_FORMAT), serv->gs->now, 0 TSRMLS_CC);
-        n = snprintf(buf, sizeof(buf), "Date: %s\r\n", date_str);
-        efree(date_str);
-        swString_append_ptr(response, buf, n);
     }
-    /**
-     * Http chunk
-     */
+    if (!(flag & HTTP_RESPONSE_CONTENT_TYPE))
+    {
+        swString_append_ptr(response, ZEND_STRL("Content-Type: text/html\r\n"));
+    }
+    if (!(flag & HTTP_RESPONSE_DATE))
+    {
+        date_str = sw_php_format_date(ZEND_STRL(SW_HTTP_DATE_FORMAT), serv->gs->now, 0 TSRMLS_CC);
+        n = snprintf(buf, l_buf, "Date: %s\r\n", date_str);
+        swString_append_ptr(response, buf, n);
+        efree(date_str);
+    }
+
     if (ctx->chunk)
     {
-        swString_append_ptr(response, SW_STRL("Transfer-Encoding: chunked\r\n") - 1);
+        if (!(flag & HTTP_RESPONSE_TRANSFER_ENCODING))
+        {
+            swString_append_ptr(response, SW_STRL("Transfer-Encoding: chunked\r\n") - 1);
+        }
     }
-    /**
-     * Content-Length
-     */
     else
+    // Content-Length
     {
 #ifdef SW_HAVE_ZLIB
-        if (ctx->gzip_enable)
+        if (ctx->enable_compression)
         {
             body_length = swoole_zlib_buffer->length;
         }
 #endif
-        n = snprintf(buf, sizeof(buf), "Content-Length: %d\r\n", body_length);
+        n = snprintf(buf, l_buf, "Content-Length: %d\r\n", body_length);
         swString_append_ptr(response, buf, n);
     }
+
     //http cookies
     if (ctx->response.zcookie)
     {
@@ -2007,15 +1968,16 @@ static void http_build_header(http_context *ctx, zval *object, swString *respons
         }
         SW_HASHTABLE_FOREACH_END();
     }
+#ifdef SW_HAVE_ZLIB
     //http compress
-    if (ctx->gzip_enable)
+    if (ctx->enable_compression)
     {
-#ifdef SW_HTTP_COMPRESS_GZIP
-        swString_append_ptr(response, SW_STRL("Content-Encoding: gzip\r\n") - 1);
-#else
-        swString_append_ptr(response, SW_STRL("Content-Encoding: deflate\r\n") - 1);
-#endif
+        const char *content_encoding = swoole_http_get_content_encoding(ctx);
+        swString_append_ptr(response, ZEND_STRL("Content-Encoding: "));
+        swString_append_ptr(response, (char*) content_encoding, strlen(content_encoding));
+        swString_append_ptr(response, ZEND_STRL("\r\n"));
     }
+#endif
     swString_append_ptr(response, ZEND_STRL("\r\n"));
     ctx->send_header = 1;
 }
@@ -2023,20 +1985,120 @@ static void http_build_header(http_context *ctx, zval *object, swString *respons
 #ifdef SW_HAVE_ZLIB
 voidpf php_zlib_alloc(voidpf opaque, uInt items, uInt size)
 {
-    return (voidpf)safe_emalloc(items, size, 0);
+    return (voidpf) safe_emalloc(items, size, 0);
 }
 
 void php_zlib_free(voidpf opaque, voidpf address)
 {
-    efree((void*)address);
+    efree((void* )address);
 }
 
-int swoole_http_response_compress(swString *body, int level)
+void swoole_http_get_compression_method(http_context *ctx, const char *accept_encoding, size_t length)
+{
+#ifdef SW_HAVE_BROTLI
+    if (swoole_strnpos((char *) accept_encoding, length, ZEND_STRL("br")) >= 0)
+    {
+        ctx->enable_compression = 1;
+        ctx->compression_level = SwooleG.serv->http_gzip_level;
+        ctx->compression_method = HTTP_COMPRESS_BR;
+    }
+    else
+#endif
+    if (swoole_strnpos((char *) accept_encoding, length, ZEND_STRL("gzip")) >= 0)
+    {
+        ctx->enable_compression = 1;
+        ctx->compression_level = SwooleG.serv->http_gzip_level;
+        ctx->compression_method = HTTP_COMPRESS_GZIP;
+    }
+    else if (swoole_strnpos((char *) accept_encoding, length, ZEND_STRL("deflate")) >= 0)
+    {
+        ctx->enable_compression = 1;
+        ctx->compression_level = SwooleG.serv->http_gzip_level;
+        ctx->compression_method = HTTP_COMPRESS_DEFLATE;
+    }
+    else
+    {
+        ctx->enable_compression = 0;
+    }
+}
+
+const char* swoole_http_get_content_encoding(http_context *ctx)
+{
+    if (ctx->compression_method == HTTP_COMPRESS_GZIP)
+    {
+       return "gzip";
+    }
+    else if (ctx->compression_method == HTTP_COMPRESS_DEFLATE)
+    {
+        return "deflate";
+    }
+#ifdef SW_HAVE_BROTLI
+    else if (ctx->compression_method == HTTP_COMPRESS_BR)
+    {
+        return "br";
+    }
+#endif
+    else
+    {
+        return NULL;
+    }
+}
+
+int swoole_http_response_compress(swString *body, int method, int level)
 {
     assert(level > 0 || level < 10);
 
-    size_t memory_size = ((size_t) ((double) body->length * (double) 1.015)) + 10 + 8 + 4 + 1;
+    int encoding;
+    //gzip: 0x1f
+    if (method == HTTP_COMPRESS_GZIP)
+    {
+        encoding = 0x1f;
+    }
+    //deflate: -0xf
+    else if (method == HTTP_COMPRESS_DEFLATE)
+    {
+        encoding = -0xf;
+    }
+#ifdef SW_HAVE_BROTLI
+    else if (method == HTTP_COMPRESS_BR)
+    {
+        if (level <= 0)
+        {
+            level = 6;
+        }
 
+        size_t memory_size = BrotliEncoderMaxCompressedSize(body->length);
+        if (memory_size > swoole_zlib_buffer->size)
+        {
+            if (swString_extend(swoole_zlib_buffer, memory_size) < 0)
+            {
+                return SW_ERR;
+            }
+        }
+
+        size_t total_out;
+        const uint8_t *next_in = (uint8_t *) body->str;
+        uint8_t *next_out = (uint8_t *) swoole_zlib_buffer->str;
+
+        if (!BrotliEncoderCompress(level, BROTLI_DEFAULT_WINDOW, BROTLI_DEFAULT_MODE, body->length, next_in, &total_out,
+                next_out))
+        {
+            return SW_ERR;
+        }
+        else
+        {
+            swoole_zlib_buffer->length = total_out;
+            return SW_OK;
+        }
+    }
+#endif
+    else
+    {
+        swWarn("Unknown compression method");
+        return SW_ERR;
+    }
+
+    size_t memory_size = ((size_t) ((double) body->length * (double) 1.015)) + 10 + 8 + 4 + 1;
     if (memory_size > swoole_zlib_buffer->size)
     {
         if (swString_extend(swoole_zlib_buffer, memory_size) < 0)
@@ -2048,18 +2110,13 @@ int swoole_http_response_compress(swString *body, int level)
     z_stream zstream;
     memset(&zstream, 0, sizeof(zstream));
 
-    //deflate: -0xf, gzip: 0x1f
-#ifdef SW_HTTP_COMPRESS_GZIP
-    int encoding = 0x1f;
-#else
-    int encoding =  -0xf;
-#endif
-
     int status;
     zstream.zalloc = php_zlib_alloc;
     zstream.zfree = php_zlib_free;
 
-    if (Z_OK == deflateInit2(&zstream, level, Z_DEFLATED, encoding, MAX_MEM_LEVEL, Z_DEFAULT_STRATEGY))
+    int retval = deflateInit2(&zstream, level, Z_DEFLATED, encoding, MAX_MEM_LEVEL, Z_DEFAULT_STRATEGY);
+
+    if (Z_OK == retval)
     {
         zstream.next_in = (Bytef *) body->str;
         zstream.next_out = (Bytef *) swoole_zlib_buffer->str;
@@ -2077,7 +2134,7 @@ int swoole_http_response_compress(swString *body, int level)
     }
     else
     {
-        swWarn("deflateInit2() failed.");
+        swWarn("deflateInit2() failed, Error: [%d].", retval);
     }
     return SW_ERR;
 }
@@ -2120,6 +2177,12 @@ static PHP_METHOD(swoole_http_response, end)
         return;
     }
 
+    http_context *ctx = http_get_context(getThis(), 0 TSRMLS_CC);
+    if (!ctx)
+    {
+        RETURN_FALSE;
+    }
+
     swString http_body;
 
     if (zdata)
@@ -2141,14 +2204,8 @@ static PHP_METHOD(swoole_http_response, end)
         http_body.str = NULL;
     }
 
-    http_context *ctx = http_get_context(getThis(), 0 TSRMLS_CC);
-    if (!ctx)
-    {
-        RETURN_FALSE;
-    }
-
 #ifdef SW_USE_HTTP2
-    if (ctx->http2)
+    if (ctx->stream)
     {
         swoole_http2_do_response(ctx, &http_body);
         RETURN_TRUE;
@@ -2169,15 +2226,15 @@ static PHP_METHOD(swoole_http_response, end)
     {
         swString_clear(swoole_http_buffer);
 #ifdef SW_HAVE_ZLIB
-        if (ctx->gzip_enable)
+        if (ctx->enable_compression)
         {
             if (http_body.length > 0)
             {
-                swoole_http_response_compress(&http_body, ctx->gzip_level);
+                swoole_http_response_compress(&http_body, ctx->compression_method, ctx->compression_level);
             }
             else
             {
-                ctx->gzip_enable = 0;
+                ctx->enable_compression = 0;
             }
         }
 #endif
@@ -2185,7 +2242,7 @@ static PHP_METHOD(swoole_http_response, end)
         if (http_body.length > 0)
         {
 #ifdef SW_HAVE_ZLIB
-            if (ctx->gzip_enable)
+            if (ctx->enable_compression)
             {
                 swString_append(swoole_http_buffer, swoole_zlib_buffer);
             }
@@ -2245,11 +2302,7 @@ static PHP_METHOD(swoole_http_response, sendfile)
     }
 
 #ifdef SW_HAVE_ZLIB
-    if (ctx->gzip_enable)
-    {
-        swoole_php_error(E_ERROR, "can't use sendfile when gzip compression is enabled.");
-        RETURN_FALSE;
-    }
+    ctx->enable_compression = 0;
 #endif
 
     if (ctx->chunk)
@@ -2412,9 +2465,7 @@ static PHP_METHOD(swoole_http_response, cookie)
         strlcat(cookie, "; httponly", len + 100);
     }
     sw_add_next_index_stringl(zcookie, cookie, strlen(cookie), 0);
-#if PHP_MAJOR_VERSION >= 7
     efree(cookie);
-#endif
 }
 
 static PHP_METHOD(swoole_http_response, rawcookie)
@@ -2520,15 +2571,15 @@ static PHP_METHOD(swoole_http_response, rawcookie)
         strlcat(cookie, "; httponly", len + 100);
     }
     sw_add_next_index_stringl(zcookie, cookie, strlen(cookie), 0);
-#if PHP_MAJOR_VERSION >= 7
     efree(cookie);
-#endif
 }
 
 static PHP_METHOD(swoole_http_response, status)
 {
     long http_status;
-    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &http_status) == FAILURE)
+    char* reason = NULL;
+    long reason_len = 0;
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l|s", &http_status, &reason, &reason_len) == FAILURE)
     {
         return;
     }
@@ -2540,6 +2591,11 @@ static PHP_METHOD(swoole_http_response, status)
     }
 
     ctx->response.status = http_status;
+    if (reason_len > 0)
+    {
+        ctx->response.reason = emalloc(32);
+        strncpy(ctx->response.reason, reason, 32 - 1);
+    }
 }
 
 static PHP_METHOD(swoole_http_response, header)
@@ -2581,11 +2637,13 @@ static PHP_METHOD(swoole_http_response, header)
         char key_buf[SW_HTTP_HEADER_KEY_SIZE];
         memcpy(key_buf, k, klen);
         key_buf[klen] = '\0';
-        if (ctx->http2)
+#ifdef SW_USE_HTTP2
+        if (ctx->stream)
         {
             swoole_strtolower(key_buf, klen);
         }
         else
+#endif
         {
             http_header_key_format(key_buf, klen);
         }
@@ -2638,11 +2696,13 @@ static PHP_METHOD(swoole_http_response, trailer)
         char key_buf[SW_HTTP_HEADER_KEY_SIZE];
         memcpy(key_buf, k, klen);
         key_buf[klen] = '\0';
-        if (ctx->http2)
+#ifdef SW_USE_HTTP2
+        if (ctx->stream)
         {
             swoole_strtolower(key_buf, klen);
         }
         else
+#endif
         {
             http_header_key_format(key_buf, klen);
         }
@@ -2655,40 +2715,11 @@ static PHP_METHOD(swoole_http_response, trailer)
 }
 #endif
 
-#ifdef SW_HAVE_ZLIB
 static PHP_METHOD(swoole_http_response, gzip)
 {
-    long level = Z_DEFAULT_COMPRESSION;
-    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &level) == FAILURE)
-    {
-        return;
-    }
-
-    http_context *context = http_get_context(getThis(), 0 TSRMLS_CC);
-    if (!context)
-    {
-        RETURN_FALSE;
-    }
-
-    if (context->send_header)
-    {
-        swoole_php_fatal_error(E_WARNING, "must be used before sending the http header.");
-        RETURN_FALSE;
-    }
-
-    if (level > 9)
-    {
-        level = 9;
-    }
-    if (level < 0)
-    {
-        level = 0;
-    }
-
-    context->gzip_enable = 1;
-    context->gzip_level = level;
+    swoole_php_error(E_DEPRECATED, "gzip() is deprecated, use option[http_compression] instead.");
+    RETURN_FALSE;
 }
-#endif
 
 static PHP_METHOD(swoole_http_response, detach)
 {
