@@ -44,10 +44,14 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_http2_client_coro_set, 0, 0, 1)
     ZEND_ARG_ARRAY_INFO(0, settings, 0)
 ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_http2_client_coro_stats, 0, 0, 1)
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_http2_client_coro_stats, 0, 0, 0)
     ZEND_ARG_INFO(0, key)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_http2_client_coro_isStreamExist, 0, 0, 1)
+    ZEND_ARG_INFO(0, stream_id)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_http2_client_coro_send, 0, 0, 1)
     ZEND_ARG_INFO(0, request)
 ZEND_END_ARG_INFO()
@@ -78,32 +82,33 @@ static PHP_METHOD(swoole_http2_client_coro, __destruct);
 static PHP_METHOD(swoole_http2_client_coro, set);
 static PHP_METHOD(swoole_http2_client_coro, connect);
 static PHP_METHOD(swoole_http2_client_coro, stats);
+static PHP_METHOD(swoole_http2_client_coro, isStreamExist);
 static PHP_METHOD(swoole_http2_client_coro, send);
 static PHP_METHOD(swoole_http2_client_coro, write);
 static PHP_METHOD(swoole_http2_client_coro, recv);
 static PHP_METHOD(swoole_http2_client_coro, goaway);
 static PHP_METHOD(swoole_http2_client_coro, close);
 
-static int http2_client_send_request(zval *zobject, zval *request TSRMLS_DC);
+static uint32_t http2_client_send_request(zval *zobject, zval *request TSRMLS_DC);
 static void http2_client_stream_free(void *ptr);
 static void http2_client_onConnect(swClient *cli);
 static void http2_client_onClose(swClient *cli);
-static void http2_client_onError(swClient *cli);
 static void http2_client_onTimeout(swTimer *timer, swTimer_node *tnode);
 static void http2_client_onReceive(swClient *cli, char *buf, uint32_t _length);
 
 static const zend_function_entry swoole_http2_client_methods[] =
 {
-    PHP_ME(swoole_http2_client_coro, __construct, arginfo_swoole_http2_client_coro_construct, ZEND_ACC_PUBLIC | ZEND_ACC_CTOR)
-    PHP_ME(swoole_http2_client_coro, __destruct,  arginfo_swoole_void, ZEND_ACC_PUBLIC | ZEND_ACC_DTOR)
-    PHP_ME(swoole_http2_client_coro, set,         arginfo_swoole_http2_client_coro_set, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_http2_client_coro, connect,     arginfo_swoole_void, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_http2_client_coro, stats,       arginfo_swoole_http2_client_coro_stats, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_http2_client_coro, send,        arginfo_swoole_http2_client_coro_send, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_http2_client_coro, write,       arginfo_swoole_http2_client_coro_write, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_http2_client_coro, recv,        arginfo_swoole_http2_client_coro_recv, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_http2_client_coro, goaway,      arginfo_swoole_http2_client_coro_goaway, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_http2_client_coro, close,       arginfo_swoole_void, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_http2_client_coro, __construct,   arginfo_swoole_http2_client_coro_construct, ZEND_ACC_PUBLIC | ZEND_ACC_CTOR)
+    PHP_ME(swoole_http2_client_coro, __destruct,    arginfo_swoole_void, ZEND_ACC_PUBLIC | ZEND_ACC_DTOR)
+    PHP_ME(swoole_http2_client_coro, set,           arginfo_swoole_http2_client_coro_set, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_http2_client_coro, connect,       arginfo_swoole_void, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_http2_client_coro, stats,         arginfo_swoole_http2_client_coro_stats, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_http2_client_coro, isStreamExist, arginfo_swoole_http2_client_coro_isStreamExist, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_http2_client_coro, send,          arginfo_swoole_http2_client_coro_send, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_http2_client_coro, write,         arginfo_swoole_http2_client_coro_write, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_http2_client_coro, recv,          arginfo_swoole_http2_client_coro_recv, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_http2_client_coro, goaway,        arginfo_swoole_http2_client_coro_goaway, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_http2_client_coro, close,         arginfo_swoole_void, ZEND_ACC_PUBLIC)
     PHP_FE_END
 };
 
@@ -154,6 +159,17 @@ void swoole_http2_client_coro_init(int module_number TSRMLS_DC)
     zend_declare_property_null(swoole_http2_response_class_entry_ptr, SW_STRL("cookies")-1, ZEND_ACC_PUBLIC TSRMLS_CC);
     zend_declare_property_null(swoole_http2_response_class_entry_ptr, SW_STRL("data")-1, ZEND_ACC_PUBLIC TSRMLS_CC);
 
+    SWOOLE_DEFINE(HTTP2_TYPE_DATA);
+    SWOOLE_DEFINE(HTTP2_TYPE_HEADERS);
+    SWOOLE_DEFINE(HTTP2_TYPE_PRIORITY);
+    SWOOLE_DEFINE(HTTP2_TYPE_RST_STREAM);
+    SWOOLE_DEFINE(HTTP2_TYPE_SETTINGS);
+    SWOOLE_DEFINE(HTTP2_TYPE_PUSH_PROMISE);
+    SWOOLE_DEFINE(HTTP2_TYPE_PING);
+    SWOOLE_DEFINE(HTTP2_TYPE_GOAWAY);
+    SWOOLE_DEFINE(HTTP2_TYPE_WINDOW_UPDATE);
+    SWOOLE_DEFINE(HTTP2_TYPE_CONTINUATION);
+
     SWOOLE_DEFINE(HTTP2_ERROR_NO_ERROR);
     SWOOLE_DEFINE(HTTP2_ERROR_PROTOCOL_ERROR);
     SWOOLE_DEFINE(HTTP2_ERROR_INTERNAL_ERROR);
@@ -197,9 +213,6 @@ static PHP_METHOD(swoole_http2_client_coro, __construct)
     context->onTimeout = NULL;
     context->coro_params = *getThis();
 
-    hcc->streams = swHashMap_new(8, http2_client_stream_free);
-    hcc->stream_id = 1;
-
     long type = SW_FLAG_ASYNC | SW_SOCK_TCP;
     if (ssl)
     {
@@ -211,11 +224,13 @@ static PHP_METHOD(swoole_http2_client_coro, __construct)
 #endif
     }
 
-    zend_update_property_long(swoole_http2_client_coro_class_entry_ptr, getThis(), ZEND_STRL("type"), type TSRMLS_CC);
-
     hcc->host = estrndup(host, host_len);
     hcc->host_len = host_len;
     hcc->port = port;
+
+    zend_update_property_long(swoole_http2_client_coro_class_entry_ptr, getThis(), ZEND_STRL("type"), type TSRMLS_CC);
+    zend_update_property_stringl(swoole_http2_client_coro_class_entry_ptr, getThis(), ZEND_STRL("host"), host, host_len TSRMLS_CC);
+    zend_update_property_long(swoole_http2_client_coro_class_entry_ptr, getThis(), ZEND_STRL("port"), port TSRMLS_CC);
 }
 
 static PHP_METHOD(swoole_http2_client_coro, set)
@@ -234,7 +249,7 @@ static PHP_METHOD(swoole_http2_client_coro, set)
     RETURN_TRUE;
 }
 
-static ssize_t http2_client_build_header(zval *zobject, zval *req, char *buffer, int buffer_len TSRMLS_DC)
+static ssize_t http2_client_build_header(zval *zobject, zval *req, char *buffer TSRMLS_DC)
 {
     char *date_str = NULL;
     int ret;
@@ -322,13 +337,6 @@ static ssize_t http2_client_build_header(zval *zobject, zval *req, char *buffer,
     size_t i;
     size_t sum = 0;
 
-#if 0
-    for (i = 0; i < index; ++i)
-    {
-        swTraceLog(SW_TRACE_HTTP2, "Header[%d]: "SW_ECHO_CYAN_BLUE"=%s", i, nv[i].name, nv[i].value);
-    }
-#endif
-
     nghttp2_hd_deflater *deflater = hcc->deflater;
     if (!deflater)
     {
@@ -347,7 +355,7 @@ static ssize_t http2_client_build_header(zval *zobject, zval *req, char *buffer,
     }
 
     buflen = nghttp2_hd_deflate_bound(deflater, nv, index);
-    if (buflen > buffer_len)
+    if (buflen > hcc->max_header_list_size)
     {
         swoole_php_error(E_WARNING, "header is too large.");
         return SW_ERR;
@@ -381,10 +389,9 @@ static ssize_t http2_client_build_header(zval *zobject, zval *req, char *buffer,
 
 static void http2_client_onReceive(swClient *cli, char *buf, uint32_t _length)
 {
-    int type = buf[3];
-    int flags = buf[4];
-    int error_code = 0;
-    int stream_id = ntohl((*(int *) (buf + 5))) & 0x7fffffff;
+    uint8_t type = buf[3];
+    uint8_t flags = buf[4];
+    uint32_t stream_id = ntohl((*(int *) (buf + 5))) & 0x7fffffff;
     uint32_t length = swHttp2_get_length(buf);
     buf += SW_HTTP2_FRAME_HEADER_SIZE;
 
@@ -398,9 +405,8 @@ static void http2_client_onReceive(swClient *cli, char *buf, uint32_t _length)
         hcc->last_stream_id = stream_id;
     }
 
-    uint16_t id;
-    uint32_t value;
-    swTraceLog(SW_TRACE_HTTP2, "["SW_ECHO_YELLOW"]\tflags=%d, stream_id=%d, length=%d", swHttp2_get_type(type), flags, stream_id, length);
+    uint16_t id = 0;
+    uint32_t value = 0;
 
     switch (type)
     {
@@ -408,6 +414,7 @@ static void http2_client_onReceive(swClient *cli, char *buf, uint32_t _length)
     {
         if (flags & SW_HTTP2_FLAG_ACK)
         {
+            swHttp2FrameTraceLog(recv, "ACK");
             return;
         }
 
@@ -415,6 +422,7 @@ static void http2_client_onReceive(swClient *cli, char *buf, uint32_t _length)
         {
             id = ntohs(*(uint16_t *) (buf));
             value = ntohl(*(uint32_t *) (buf + sizeof(uint16_t)));
+            swHttp2FrameTraceLog(recv, "id=%d, value=%d", id, value);
             switch (id)
             {
             case SW_HTTP2_SETTING_HEADER_TABLE_SIZE:
@@ -446,34 +454,45 @@ static void http2_client_onReceive(swClient *cli, char *buf, uint32_t _length)
         }
 
         swHttp2_set_frame_header(frame, SW_HTTP2_TYPE_SETTINGS, 0, SW_HTTP2_FLAG_ACK, stream_id);
-        swTraceLog(SW_TRACE_HTTP2, "["SW_ECHO_GREEN", ACK, STREAM#%d]\t[length=%d]", swHttp2_get_type(SW_HTTP2_TYPE_SETTINGS), stream_id, length);
         cli->send(cli, frame, SW_HTTP2_FRAME_HEADER_SIZE, 0);
         return;
     }
     case SW_HTTP2_TYPE_WINDOW_UPDATE:
     {
-        hcc->send_window = ntohl(*(int *) buf);
-        swTraceLog(SW_TRACE_HTTP2, "update: send_window=%d.", hcc->recv_window);
+        value = ntohl(*(uint32_t *) buf);
+        swHttp2FrameTraceLog(recv, "window_size_increment=%d", value);
+        if (stream_id == 0)
+        {
+            hcc->send_window += value;
+        }
+        else
+        {
+            http2_client_stream *stream = swHashMap_find_int(hcc->streams, stream_id);
+            if (stream)
+            {
+                stream->send_window += value;
+            }
+        }
         return;
     }
     case SW_HTTP2_TYPE_PING:
     {
+        swHttp2FrameTraceLog(recv, "ping");
         swHttp2_set_frame_header(frame, SW_HTTP2_TYPE_PING, SW_HTTP2_FRAME_PING_PAYLOAD_SIZE, SW_HTTP2_FLAG_ACK, stream_id);
         memcpy(frame + SW_HTTP2_FRAME_HEADER_SIZE, buf + SW_HTTP2_FRAME_HEADER_SIZE, SW_HTTP2_FRAME_PING_PAYLOAD_SIZE);
-        swTraceLog(SW_TRACE_HTTP2, "["SW_ECHO_GREEN", STREAM#%d]", swHttp2_get_type(SW_HTTP2_TYPE_PING), stream_id);
         cli->send(cli, frame, SW_HTTP2_FRAME_HEADER_SIZE + SW_HTTP2_FRAME_PING_PAYLOAD_SIZE, 0);
         return;
     }
     case SW_HTTP2_TYPE_GOAWAY:
     {
-        uint32_t server_last_stream_id = htonl(*(int *) (buf));
+        uint32_t server_last_stream_id = ntohl(*(uint32_t *) (buf));
         buf += 4;
-        error_code = htonl(*(int *) (buf));
+        value = ntohl(*(uint32_t *) (buf));
         buf += 4;
+        swHttp2FrameTraceLog(recv, "last_stream_id=%d, error_code=%d, opaque_data=[%.*s]", server_last_stream_id, value, length - SW_HTTP2_GOAWAY_SIZE, buf);
 
         // update goaway error code and error msg
-        swTraceLog(SW_TRACE_HTTP2, "["SW_ECHO_RED"] last_stream_id=%d, error_code=%d, opaque_data=[%.*s]", "GOAWAY", server_last_stream_id, error_code, length - SW_HTTP2_GOAWAY_SIZE, buf);
-        zend_update_property_long(swoole_http2_client_coro_class_entry_ptr, zobject, ZEND_STRL("errCode"), error_code TSRMLS_CC);
+        zend_update_property_long(swoole_http2_client_coro_class_entry_ptr, zobject, ZEND_STRL("errCode"), value TSRMLS_CC);
         zend_update_property_stringl(swoole_http2_client_coro_class_entry_ptr, zobject, ZEND_STRL("errMsg"), buf, length - SW_HTTP2_GOAWAY_SIZE TSRMLS_CC);
         zend_update_property_long(swoole_http2_client_coro_class_entry_ptr, zobject, ZEND_STRL("serverLastStreamId"), server_last_stream_id TSRMLS_CC);
 
@@ -501,15 +520,20 @@ static void http2_client_onReceive(swClient *cli, char *buf, uint32_t _length)
     }
     case SW_HTTP2_TYPE_RST_STREAM:
     {
-        error_code = htonl(*(int *) (buf));
-        swTraceLog(SW_TRACE_HTTP2, "["SW_ECHO_RED"] stream_id=%d, error_code=%d.", "RST_STREAM", stream_id, error_code);
+        value = ntohl(*(uint32_t *) (buf));
+        swHttp2FrameTraceLog(recv, "error_code=%d", value);
 
+        // FIXME stream leak?
         if (hcc->iowait == 0)
         {
             return;
         }
         break;
     }
+    default:
+    {
+        swHttp2FrameTraceLog(recv, "");
+    }
     }
 
     http2_client_stream *stream = swHashMap_find_int(hcc->streams, stream_id);
@@ -546,6 +570,7 @@ static void http2_client_onReceive(swClient *cli, char *buf, uint32_t _length)
             }
 
             // now we control the connection flow only (not stream)
+            // our window size is unlimited, so we don't worry about subtraction overflow
             hcc->recv_window -= length;
             stream->recv_window -= length;
             if (hcc->recv_window < (SW_HTTP2_MAX_WINDOW_SIZE / 4))
@@ -575,7 +600,7 @@ static void http2_client_onReceive(swClient *cli, char *buf, uint32_t _length)
         if (type == SW_HTTP2_TYPE_RST_STREAM)
         {
             zend_update_property_long(swoole_http2_response_class_entry_ptr, zresponse, ZEND_STRL("statusCode"), -3 TSRMLS_CC);
-            zend_update_property_long(swoole_http2_response_class_entry_ptr, zresponse, ZEND_STRL("errCode"), error_code TSRMLS_CC);
+            zend_update_property_long(swoole_http2_response_class_entry_ptr, zresponse, ZEND_STRL("errCode"), value TSRMLS_CC);
         }
         else if (stream_type == SW_HTTP2_STREAM_PIPELINE && !(flags & SW_HTTP2_FLAG_END_STREAM))
         {
@@ -644,7 +669,7 @@ static void http2_client_stream_free(void *ptr)
     efree(stream);
 }
 
-static int http2_client_send_request(zval *zobject, zval *req TSRMLS_DC)
+static uint32_t http2_client_send_request(zval *zobject, zval *req TSRMLS_DC)
 {
     http2_client_property *hcc = swoole_get_property(zobject, HTTP2_CLIENT_CORO_PROPERTY);
     swClient *cli = hcc->client;
@@ -663,12 +688,12 @@ static int http2_client_send_request(zval *zobject, zval *req TSRMLS_DC)
     /**
      * send header
      */
-    char buffer[8192];
-    ssize_t n = http2_client_build_header(zobject, req, buffer + SW_HTTP2_FRAME_HEADER_SIZE, sizeof(buffer) - SW_HTTP2_FRAME_HEADER_SIZE TSRMLS_CC);
+    char* buffer = SwooleTG.buffer_stack->str;
+    ssize_t n = http2_client_build_header(zobject, req, buffer + SW_HTTP2_FRAME_HEADER_SIZE TSRMLS_CC);
     if (n <= 0)
     {
         swWarn("http2_client_build_header() failed.");
-        return -1;
+        return 0;
     }
 
     // malloc
@@ -683,6 +708,9 @@ static int http2_client_send_request(zval *zobject, zval *req TSRMLS_DC)
     stream->send_window = SW_HTTP2_DEFAULT_WINDOW_SIZE;
     stream->recv_window = SW_HTTP2_DEFAULT_WINDOW_SIZE;
 
+    // add to map
+    swHashMap_add_int(hcc->streams, stream->stream_id, stream);
+
     if (ZVAL_IS_NULL(post_data))
     {
         //pipeline
@@ -692,7 +720,7 @@ static int http2_client_send_request(zval *zobject, zval *req TSRMLS_DC)
         }
         else
         {
-            swHttp2_set_frame_header(buffer, SW_HTTP2_TYPE_HEADERS, n, SW_HTTP2_FLAG_END_STREAM | SW_HTTP2_FLAG_END_HEADERS, hcc->stream_id);
+            swHttp2_set_frame_header(buffer, SW_HTTP2_TYPE_HEADERS, n, SW_HTTP2_FLAG_END_STREAM | SW_HTTP2_FLAG_END_HEADERS, stream->stream_id);
         }
     }
     else
@@ -702,9 +730,6 @@ static int http2_client_send_request(zval *zobject, zval *req TSRMLS_DC)
 
     zend_update_property_long(swoole_http2_response_class_entry_ptr, stream->response_object, ZEND_STRL("streamId"), stream->stream_id TSRMLS_CC);
 
-    // add to map
-    swHashMap_add_int(hcc->streams, stream->stream_id, stream);
-
     swTraceLog(SW_TRACE_HTTP2, "["SW_ECHO_GREEN", STREAM#%d] length=%zd", swHttp2_get_type(SW_HTTP2_TYPE_HEADERS), stream->stream_id, n);
     cli->send(cli, buffer, n + SW_HTTP2_FRAME_HEADER_SIZE, 0);
 
@@ -713,35 +738,64 @@ static int http2_client_send_request(zval *zobject, zval *req TSRMLS_DC)
      */
     if (!ZVAL_IS_NULL(post_data))
     {
+        char *p;
+        zend_size_t len;
+        smart_str formstr_s = { NULL, 0 };
+        uint8_t send_flag;
+        uint32_t send_len;
+
         int flag = stream->type == SW_HTTP2_STREAM_PIPELINE ? 0 : SW_HTTP2_FLAG_END_STREAM;
         if (Z_TYPE_P(post_data) == IS_ARRAY)
         {
-            zend_size_t len;
-            smart_str formstr_s = { 0 };
-            char *formstr = sw_http_build_query(post_data, &len, &formstr_s TSRMLS_CC);
-            if (formstr == NULL)
+            p = sw_http_build_query(post_data, &len, &formstr_s TSRMLS_CC);
+            if (p == NULL)
             {
                 swoole_php_error(E_WARNING, "http_build_query failed.");
-                return -1;
+                return 0;
             }
-            memset(buffer, 0, SW_HTTP2_FRAME_HEADER_SIZE);
-            swHttp2_set_frame_header(buffer, SW_HTTP2_TYPE_DATA, len, flag, stream->stream_id);
-            swTraceLog(SW_TRACE_HTTP2, "["SW_ECHO_GREEN", END, STREAM#%d] length=%zu", swHttp2_get_type(SW_HTTP2_TYPE_DATA), stream->stream_id, len);
-            cli->send(cli, buffer, SW_HTTP2_FRAME_HEADER_SIZE, 0);
-            cli->send(cli, formstr, len, 0);
-            smart_str_free(&formstr_s);
         }
         else
         {
             convert_to_string(post_data);
-            swHttp2_set_frame_header(buffer, SW_HTTP2_TYPE_DATA, Z_STRLEN_P(post_data), flag, stream->stream_id);
-            swTraceLog(SW_TRACE_HTTP2, "["SW_ECHO_GREEN", END, STREAM#%d] length=%zu", swHttp2_get_type(SW_HTTP2_TYPE_DATA), stream->stream_id, Z_STRLEN_P(post_data));
-            cli->send(cli, buffer, SW_HTTP2_FRAME_HEADER_SIZE, 0);
-            cli->send(cli, Z_STRVAL_P(post_data), Z_STRLEN_P(post_data), 0);
+            p = Z_STRVAL_P(post_data);
+            len = Z_STRLEN_P(post_data);
+        }
+
+        swTraceLog(SW_TRACE_HTTP2, "["SW_ECHO_GREEN", END, STREAM#%d] length=%zu", swHttp2_get_type(SW_HTTP2_TYPE_DATA), stream->stream_id, len);
+
+        while (len > 0)
+        {
+            if (len > hcc->max_frame_size)
+            {
+                send_len = hcc->max_frame_size;
+                send_flag = 0;
+            }
+            else
+            {
+                send_len = len;
+                send_flag = flag;
+            }
+            swHttp2_set_frame_header(buffer, SW_HTTP2_TYPE_DATA, send_len, send_flag, stream->stream_id);
+            if (cli->send(cli, buffer, SW_HTTP2_FRAME_HEADER_SIZE, 0) < 0)
+            {
+                return 0;
+            }
+            if (cli->send(cli, p, send_len, 0) < 0)
+            {
+                return 0;
+            }
+            len -= send_len;
+            p += send_len;
+        }
+
+        if (formstr_s.s)
+        {
+            smart_str_free(&formstr_s);
         }
     }
 
     hcc->stream_id += 2;
+
     return stream->stream_id;
 }
 
@@ -817,8 +871,8 @@ static PHP_METHOD(swoole_http2_client_coro, send)
         goto error;
     }
 
-    int stream_id = http2_client_send_request(getThis(), request TSRMLS_CC);
-    if (stream_id < 0)
+    uint32_t stream_id = http2_client_send_request(getThis(), request TSRMLS_CC);
+    if (stream_id == 0)
     {
         RETURN_FALSE;
     }
@@ -875,6 +929,7 @@ static void http2_client_onConnect(swClient *cli)
 
     hcc->ready = 1;
     hcc->stream_id = 1;
+    hcc->streams = swHashMap_new(8, http2_client_stream_free);
     http2_client_send_setting(cli);
 
     // [init]: we must set default value, server is not always send all the settings
@@ -882,7 +937,7 @@ static void http2_client_onConnect(swClient *cli)
     hcc->send_window = SW_HTTP2_DEFAULT_WINDOW_SIZE;
     hcc->recv_window = SW_HTTP2_DEFAULT_WINDOW_SIZE;
     hcc->max_frame_size = SW_HTTP2_MAX_FRAME_SIZE;
-    // hcc->max_header_list_size = 1; unknown
+    hcc->max_header_list_size = SW_HTTP2_MAX_HEADER_LIST_SIZE;
 
     hcc->iowait = 0;
     zval *result;
@@ -900,67 +955,39 @@ static void http2_client_onConnect(swClient *cli)
 static void http2_client_onClose(swClient *cli)
 {
     zval *zobject = cli->object;
-    http2_client_property *hcc = swoole_get_property(zobject, HTTP2_CLIENT_CORO_PROPERTY);
-
     zend_update_property_bool(swoole_http2_client_coro_class_entry_ptr, zobject, ZEND_STRL("connected"), 0 TSRMLS_CC);
-
     php_swoole_client_free(zobject, cli TSRMLS_CC);
 
+    http2_client_property *hcc = swoole_get_property(zobject, HTTP2_CLIENT_CORO_PROPERTY);
     if (!hcc)
     {
         return;
     }
     hcc->client = NULL;
+    if (hcc->streams)
+    {
+        swHashMap_free(hcc->streams);
+        hcc->streams = NULL;
+    }
     if (hcc->iowait == 0)
     {
         return;
     }
-
     hcc->cid = 0;
     hcc->iowait = 0;
-    zval *result;
-    SW_MAKE_STD_ZVAL(result);
-    ZVAL_BOOL(result, 0);
+
+    zval _result;
+    zval *result = &_result;
     zval *retval = NULL;
+    ZVAL_FALSE(result);
     php_context *context = swoole_get_property(zobject, HTTP2_CLIENT_CORO_CONTEXT);
     int ret = coro_resume(context, result, &retval);
     if (ret == CORO_END && retval)
     {
-        sw_zval_ptr_dtor(&retval);
+        zval_ptr_dtor(retval);
     }
 }
 
-static void http2_client_onError(swClient *cli)
-{
-    zval zdata;
-    zval *retval = NULL;
-    zval *zobject = cli->object;
-    http2_client_property *hcc = swoole_get_property(zobject, HTTP2_CLIENT_CORO_PROPERTY);
-
-    php_swoole_client_free(zobject, cli TSRMLS_CC);
-
-    if (!hcc)
-    {
-        return;
-    }
-    hcc->client = NULL;
-    if (hcc->iowait == 0)
-    {
-        return;
-    }
-
-    hcc->cid = 0;
-    hcc->iowait = 0;
-    php_context *context = swoole_get_property(zobject, HTTP2_CLIENT_CORO_CONTEXT);
-    ZVAL_BOOL(&zdata, 0); //return false
-    coro_resume(context, &zdata, &retval);
-    if (retval != NULL)
-    {
-        sw_zval_ptr_dtor(&retval);
-    }
-    zval_ptr_dtor(&zdata);
-}
-
 static void http2_client_onTimeout(swTimer *timer, swTimer_node *tnode)
 {
     php_context *ctx = tnode->data;
@@ -993,20 +1020,19 @@ static PHP_METHOD(swoole_http2_client_coro, __destruct)
         if (hcc->inflater)
         {
             nghttp2_hd_inflate_del(hcc->inflater);
-            hcc->inflater = NULL;
         }
         if (hcc->deflater)
         {
             nghttp2_hd_deflate_del(hcc->deflater);
-            hcc->deflater = NULL;
         }
         if (hcc->host)
         {
             efree(hcc->host);
-            hcc->host = NULL;
         }
-
-        swHashMap_free(hcc->streams);
+        if (hcc->streams)
+        {
+            swHashMap_free(hcc->streams);
+        }
         efree(hcc);
         swoole_set_property(zobject, HTTP2_CLIENT_CORO_PROPERTY, NULL);
     }
@@ -1086,7 +1112,7 @@ static PHP_METHOD(swoole_http2_client_coro, connect)
 
     cli->onConnect = http2_client_onConnect;
     cli->onClose = http2_client_onClose;
-    cli->onError = http2_client_onError;
+    cli->onError = http2_client_onClose; // same as close
     cli->onReceive = http2_client_onReceive;
     cli->http2 = 1;
 
@@ -1147,7 +1173,7 @@ static PHP_METHOD(swoole_http2_client_coro, stats)
         }
         else if (strcmp(key.str, "active_stream_num") == 0)
         {
-           RETURN_LONG(swHashMap_count(hcc->streams));
+           RETURN_LONG(hcc->streams ? swHashMap_count(hcc->streams) : 0);
         }
     }
     else
@@ -1160,10 +1186,41 @@ static PHP_METHOD(swoole_http2_client_coro, stats)
         sw_add_assoc_long_ex(return_value, ZEND_STRS("max_concurrent_streams"), hcc->max_concurrent_streams);
         sw_add_assoc_long_ex(return_value, ZEND_STRS("max_frame_size"), hcc->max_frame_size);
         sw_add_assoc_long_ex(return_value, ZEND_STRS("max_header_list_size"), hcc->max_header_list_size);
-        sw_add_assoc_long_ex(return_value, ZEND_STRS("active_stream_num"), swHashMap_count(hcc->streams));
+        sw_add_assoc_long_ex(return_value, ZEND_STRS("active_stream_num"), hcc->streams ? swHashMap_count(hcc->streams) : 0);
     }
 }
 
+static PHP_METHOD(swoole_http2_client_coro, isStreamExist)
+{
+    zend_long stream_id = 0;
+    if (zend_parse_parameters(ZEND_NUM_ARGS()TSRMLS_CC, "l", &stream_id) == FAILURE)
+    {
+        RETURN_FALSE;
+    }
+    if (stream_id < 0)
+    {
+        RETURN_FALSE;
+    }
+    http2_client_property *hcc = swoole_get_property(getThis(), HTTP2_CLIENT_CORO_PROPERTY);
+    if (!hcc->client)
+    {
+        RETURN_FALSE;
+    }
+    else
+    {
+        if (stream_id == 0)
+        {
+            RETURN_TRUE;
+        }
+        if (!hcc->streams)
+        {
+            RETURN_FALSE;
+        }
+    }
+    http2_client_stream *stream = swHashMap_find_int(hcc->streams, stream_id);
+    RETURN_BOOL(stream ? 1 : 0);
+}
+
 static PHP_METHOD(swoole_http2_client_coro, write)
 {
     http2_client_property *hcc = swoole_get_property(getThis(), HTTP2_CLIENT_CORO_PROPERTY);
@@ -1201,7 +1258,8 @@ static PHP_METHOD(swoole_http2_client_coro, goaway)
     int ret;
     char* frame;
     uint8_t error_code = SW_HTTP2_ERROR_NO_ERROR;
-    swString debug_data;
+    char* debug_data = NULL;
+    long  debug_data_len = 0;
 
     if (!cli || !cli->socket || cli->socket->closed)
     {
@@ -1209,19 +1267,18 @@ static PHP_METHOD(swoole_http2_client_coro, goaway)
         RETURN_FALSE;
     }
 
-    bzero(&debug_data, sizeof(debug_data));
-    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|ls", &error_code, &debug_data.str, &debug_data.length) == FAILURE)
+    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|ls", &error_code, &debug_data, &debug_data_len) == FAILURE)
     {
         return;
     }
 
-    size_t length = SW_HTTP2_FRAME_HEADER_SIZE + SW_HTTP2_GOAWAY_SIZE + debug_data.length;
+    size_t length = SW_HTTP2_FRAME_HEADER_SIZE + SW_HTTP2_GOAWAY_SIZE + debug_data_len;
     frame = emalloc(length);
     bzero(frame, length);
-    swHttp2_set_frame_header(frame, SW_HTTP2_TYPE_GOAWAY, SW_HTTP2_GOAWAY_SIZE + debug_data.length, error_code, 0);
+    swHttp2_set_frame_header(frame, SW_HTTP2_TYPE_GOAWAY, SW_HTTP2_GOAWAY_SIZE + debug_data_len, error_code, 0);
     *(uint32_t*) (frame + SW_HTTP2_FRAME_HEADER_SIZE) = htonl(hcc->last_stream_id);
     *(uint32_t*) (frame + SW_HTTP2_FRAME_HEADER_SIZE + 4) = htonl(error_code);
-    memcpy(frame + SW_HTTP2_FRAME_HEADER_SIZE + SW_HTTP2_GOAWAY_SIZE, debug_data.str, debug_data.length);
+    memcpy(frame + SW_HTTP2_FRAME_HEADER_SIZE + SW_HTTP2_GOAWAY_SIZE, debug_data, debug_data_len);
     swTraceLog(SW_TRACE_HTTP2, "["SW_ECHO_GREEN"] Send: last-sid=%d, error-code=%d", swHttp2_get_type(SW_HTTP2_TYPE_GOAWAY), hcc->last_stream_id, error_code);
 
     ret = cli->send(cli, frame, length, 0);
