@@ -550,14 +550,10 @@ static PHP_METHOD(swoole_process, signal)
     }
     signal_callback[signo] = callback;
 
-#if PHP_MAJOR_VERSION >= 7 || (PHP_MAJOR_VERSION >= 5 && PHP_MINOR_VERSION >= 4)
     /**
      * use user settings
      */
     SwooleG.use_signalfd = SwooleG.enable_signalfd;
-#else
-    SwooleG.use_signalfd = 0;
-#endif
 
     swSignal_add(signo, handler);
 
@@ -820,9 +816,7 @@ static PHP_METHOD(swoole_process, read)
     }
     buf[ret] = 0;
     SW_ZVAL_STRINGL(return_value, buf, ret, 0);
-#if PHP_MAJOR_VERSION >= 7
     efree(buf);
-#endif
 }
 
 static PHP_METHOD(swoole_process, write)
