@@ -86,9 +86,7 @@ void swoole_server_port_init(int module_number TSRMLS_DC)
     zend_declare_property_long(swoole_server_port_class_entry_ptr, ZEND_STRL("sock"), 0, ZEND_ACC_PUBLIC TSRMLS_CC);
     zend_declare_property_null(swoole_server_port_class_entry_ptr, ZEND_STRL("setting"), ZEND_ACC_PUBLIC TSRMLS_CC);
 
-#ifdef HAVE_PCRE
     zend_declare_property_null(swoole_server_port_class_entry_ptr, ZEND_STRL("connections"), ZEND_ACC_PUBLIC TSRMLS_CC);
-#endif
 }
 
 static PHP_METHOD(swoole_server_port, __construct)
@@ -267,6 +265,11 @@ static PHP_METHOD(swoole_server_port, set)
         port->websocket_subprotocol = sw_strdup(Z_STRVAL_P(v));
         port->websocket_subprotocol_length = Z_STRLEN_P(v);
     }
+    if (php_swoole_array_get_value(vht, "open_websocket_close_frame", v))
+    {
+        convert_to_boolean(v);
+        port->open_websocket_close_frame = Z_BVAL_P(v);
+    }
 #ifdef SW_USE_HTTP2
     //http2 protocol
     if (php_swoole_array_get_value(vht, "open_http2_protocol", v))
@@ -553,6 +556,7 @@ static PHP_METHOD(swoole_server_port, on)
         "Receive",
         "Close",
         "Packet",
+        NULL, //onStart
         NULL,
         NULL,
         NULL,
@@ -561,9 +565,8 @@ static PHP_METHOD(swoole_server_port, on)
         NULL,
         NULL,
         NULL,
-        NULL,
-        NULL,
-        NULL,
+        NULL, //onManagerStop
+        NULL, //onPipeMessage
         "Request",
         "HandShake",
         "Open",
