@@ -30,10 +30,8 @@ typedef struct _swTimer_callback
 {
     zval* callback;
     zval* data;
-#if PHP_MAJOR_VERSION >= 7
     zval _callback;
     zval _data;
-#endif
 #ifdef SW_COROUTINE
     zend_fcall_info_cache *func_cache;
 #endif
@@ -49,7 +47,6 @@ void php_swoole_clear_all_timer()
     {
         return;
     }
-    SWOOLE_GET_TSRMLS;
     uint64_t timer_id;
     //kill user process
     while (1)
@@ -304,7 +301,7 @@ void php_swoole_onInterval(swTimer *timer, swTimer_node *tnode)
 
 void php_swoole_check_timer(int msec)
 {
-    if (SwooleG.timer.fd == 0)
+    if (unlikely(SwooleG.timer.fd == 0))
     {
         swTimer_init(msec);
     }
